# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestHostIndicators:
    """Test TcEx Host Indicators."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti

    def test_host_get(self, hostname='www.hostname-title-42353.com'):
        """Test host get."""
        # create
        self.host_create(hostname)

        # get
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('hostName') == hostname

        # delete
        self.host_delete(hostname)

    def test_host_get_attributes(self, hostname='www.hostname-title-12453.com'):
        """Test host get."""
        # create
        self.host_create(hostname)
        self.test_host_add_attribute(False, hostname, 'Description', 'test1')
        self.test_host_add_attribute(False, hostname, 'Description', 'test2')
        self.test_host_add_attribute(False, hostname, 'Description', 'test3')

        # get attributes
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

        # delete
        self.host_delete(hostname)

    def test_host_get_tags(self, hostname='www.hostname-title-64235.com'):
        """Test host get."""
        # create
        self.host_create(hostname)
        self.test_host_add_tag(False, hostname, 'One')
        self.test_host_add_tag(False, hostname, 'Two')

        # get tags
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

        # delete
        self.host_delete(hostname)

    def test_host_get_include(self, hostname='www.hostname-title-78159.com'):
        """Test host get."""
        self.host_create(hostname)
        self.test_host_add_attribute(False, hostname, 'Description', 'test123')
        self.test_host_add_label(False, hostname, 'TLP:RED')
        self.test_host_add_tag(False, hostname, 'PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('host').get('hostName') == hostname
        assert ti_data.get('data').get('host').get('attribute')[0].get('value') == 'test123'
        assert ti_data.get('data').get('host').get('securityLabel')[0].get('name') == 'TLP:RED'
        assert ti_data.get('data').get('host').get('tag')[0].get('name') == 'PyTest'

        # delete
        self.host_delete(hostname)

    def host_create(self, hostname='www.hostname-title-65341.com'):
        """Test host create."""
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('host').get('hostName') == hostname

    def test_host_add_attribute(
        self,
        should_create=True,
        hostname='www.hostname-title-nkjvb.com',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test host attribute add."""
        if should_create:
            self.host_create(hostname)

        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value
        if should_create:
            self.host_delete(hostname)

    def test_host_add_label(
        self, should_create=True, hostname='www.hostname-title-ds4vb.com', label='TLP:GREEN'
    ):
        """Test host attribute add."""
        if should_create:
            self.host_create(hostname)

        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'
        if should_create:
            self.host_delete(hostname)

    def test_host_add_tag(
        self, should_create=True, hostname='www.hostname-title-fdsv23.com', name='Crimeware'
    ):
        """Test host attribute add."""
        if should_create:
            self.host_create(hostname)

        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.add_tag(name=name)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'
        if should_create:
            self.host_delete(hostname)

    def host_delete(self, hostname='www.hostname-title-523fa.com'):
        """Test host delete."""
        # create indicator
        self.host_create(hostname)

        # delete indicator
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner)
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_host_update(self, hostname='www.hostname-title-b3da3.com'):
        """Test host update."""
        # create indicator
        self.host_create(hostname)

        # update indicator
        ti = self.ti.host(hostname, owner=tcex.args.tc_owner, rating=5, confidence=10)
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('host').get('rating') == 5.0
        assert ti_data.get('data').get('host').get('confidence') == 10

        # delete indicator
        self.host_delete(hostname)



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestEmbedded:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""
        out_variables = []
        setattr(tcex.args, 'tc_playbook_out_variables', ','.join(out_variables))

        # add String inputs
        string_inputs = [
            {'variable': '#App:0001:string.1!String', 'data': 'DATA'},
            {'variable': '#App:0001:string.2!String', 'data': 'This is \"a string\"'},
            {'variable': '#App:0001:string.3!String', 'data': 'two'},
            {'variable': '#App:0001:string.4!String', 'data': 'one\ntwo\n'},
            {'variable': '#App:0001:string.5!String', 'data': r'\snow or later\s'},
            {'variable': '#App:0001:string.6!String', 'data': r'\snow is not \\snow.\s'},
        ]
        for i in string_inputs:
            tcex.playbook.create_string(i.get('variable'), i.get('data'))

        # add StringArray inputs
        string_array_inputs = [
            {
                'variable': '#App:0001:array.1!StringArray',
                'data': ['#App:0001:string.3!String', 'three'],
            },
            {
                'variable': '#App:0001:array.2!StringArray',
                'data': ['#App:0001:string.4!String', 'three\nfour\n'],
            },
        ]
        for i in string_array_inputs:
            tcex.playbook.create_string_array(i.get('variable'), i.get('data'))

        # add KeyValue inputs
        keyvalue_inputs = [
            {
                'variable': '#App:0001:keyvalue.1!KeyValue',
                'data': {'key': 'kv1', 'value': '#App:0001:string.3!String'},
            },
            {
                'variable': '#App:0001:keyvalue.2!KeyValue',
                'data': {'key': 'kv1', 'value': '#App:0001:array.1!StringArray'},
            },
            {
                'variable': '#App:0001:keyvalue.3!KeyValue',
                'data': {'key': 'kv1', 'value': '#App:0001:array.2!StringArray'},
            },
        ]
        for i in keyvalue_inputs:
            tcex.playbook.create_key_value(i.get('variable'), i.get('data'))

        # add KeyValueArray inputs
        keyvalue_array_inputs = [
            {
                'variable': '#App:0001:keyvalue.array.1!KeyValueArray',
                'data': [
                    {'key': 'kv.string', 'value': '#App:0001:string.3!String'},
                    {'key': 'kv.string.array', 'value': '#App:0001:array.1!StringArray'},
                ],
            }
        ]
        for i in keyvalue_array_inputs:
            tcex.playbook.create_key_value_array(i.get('variable'), i.get('data'))

    @pytest.mark.parametrize(
        'embedded_value,resolved_value',
        [
            # test \s replacement
            (r'\stest', r' test'),
            (r'test\s', r'test '),
            (r'\stest\s', r' test '),
            (r'\\stest', r'\stest'),
            (r'test\\s', r'test\s'),
            (r'\\\somedir', r'\\somedir'),
            (r'\snow\sor\slater\s', ' now or later '),
            (
                '{"numbers": "#App:0001:string.4!String three\n\r"}',
                '{"numbers": "one\ntwo\n three\n\r"}',
            ),
            # String Test: new lines
            ('#App:0001:string.4!String', 'one\ntwo\n'),
            # String Test: escaped \n
            ('#App:0001:string.5!String', r'\snow or later\s'),
            # String Test: \s replacement
            ('#App:0001:string.6!String', r'\snow is not \\snow.\s'),
            # String Test: String in String
            ('This is #App:0001:string.1!String in a String.', 'This is DATA in a String.'),
            # String Test: StringArray in String
            (
                'This is #App:0001:array.1!StringArray in a String.',
                'This is ["two", "three"] in a String.',
            ),
            # String Test: KeyValue in String
            (
                'This is #App:0001:keyvalue.1!KeyValue in a String.',
                'This is {"value": "two", "key": "kv1"} in a String.',
            ),
            # String Test: KeyValueArray in String
            (
                'This is #App:0001:keyvalue.array.1!KeyValueArray in a String.',
                'This is [{"value": "two", "key": "kv.string"}, {"value": ["two", "three"], '
                '"key": "kv.string.array"}] in a String.',
            ),
            # String Test: String and StringArray in String
            (
                'This is #App:0001:string.1!String and #App:0001:array.1!StringArray in '
                'a String.',
                'This is DATA and ["two", "three"] in a String.',
            ),
            # String Test: service now use case
            ('{"work_notes": "#App:0001:string.4!String"}', '{"work_notes": "one\ntwo\n"}'),
            # KeyValueArray Test: Nested KeyValue
            (
                '{"key": "one", "value": #App:0001:keyvalue.1!KeyValue}',
                '{"key": "one", "value": {"value": "two", "key": "kv1"}}',
            ),
            (
                (
                    '{'
                    '    "KeyValue": #App:0001:keyvalue.1!KeyValue,'
                    '    "KeyValueArray": #App:0001:keyvalue.array.1!KeyValueArray,'
                    '    "String": "#App:0001:string.1!String",'
                    '    "StringArray": #App:0001:array.1!StringArray'
                    '}'
                ),
                (
                    '{'
                    '    "KeyValue": {"value": "two", "key": "kv1"},'
                    '    "KeyValueArray": '
                    '[{"value": "two", "key": "kv.string"}, '
                    '{"value": ["two", "three"], "key": "kv.string.array"}],'
                    '    "String": "DATA",'
                    '    "StringArray": ["two", "three"]'
                    '}'
                ),
            ),
            # StringArray Test: String in StringArray
            ('#App:0001:array.1!StringArray', ['two', 'three']),
        ],
    )
    def test_embedded_read_string(self, embedded_value, resolved_value):
        """Test playbook embedded string in string"""
        assert tcex.playbook.read(embedded_value) == resolved_value

        # print('redis    : <{}>'.format(tcex.playbook.read(embedded_value)))
        # print(' resolved : <{}>'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(embedded_value))))
        # print('resolved (type): {}'.format(type(resolved_value)))

    @pytest.mark.parametrize(
        'variable,embedded_value,resolved_value',
        [
            (
                '#App:0001:embedded.string.1!String',
                'Test of embedded #App:0001:string.1!String in String.',
                'Test of embedded DATA in String.',
            ),
            (
                '#App:0001:embedded.string.1!String',
                '#App:0001:string.2!String inside a string.',
                r'This is \"a string\" inside a string.',
            ),
            (
                '#App:0001:embedded.string.1!String',
                '{"work_notes": "#App:0001:string.4!String"}',
                '{"work_notes": "one\ntwo\n"}',
            ),
        ],
    )
    def test_embedded_string_in_string(self, variable, embedded_value, resolved_value):
        """Test playbook embedded string in string"""
        tcex.playbook.create_string(variable, embedded_value)
        assert tcex.playbook.read(variable) == resolved_value

        # print('redis    : {}'.format(tcex.playbook.read(variable)))
        # print('resolved : {}'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(variable))))
        # print('resolved (type): {}'.format(type(resolved_value)))

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,embedded_value,resolved_value',
        [
            (
                '#App:0001:embedded.string.1!String',
                'Test of embedded #App:0001:array.1!StringArray in String.',
                'Test of embedded ["two", "three"] in String.',
            )
        ],
    )
    def test_embedded_string_array_in_string(self, variable, embedded_value, resolved_value):
        """Test playbook embedded string array in string"""
        tcex.playbook.create_string(variable, embedded_value)
        assert tcex.playbook.read(variable) == resolved_value

        # print('redis    : {}'.format(tcex.playbook.read(variable)))
        # print('resolved : {}'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(variable))))
        # print('resolved (type): {}'.format(type(resolved_value)))

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,embedded_value,resolved_value',
        [
            (
                '#App:0001:embedded.string_array.2!StringArray',
                ['one', '#App:0001:string.3!String', 'three'],
                ['one', 'two', 'three'],
            )
        ],
    )
    def test_embedded_string_in_string_array(self, variable, embedded_value, resolved_value):
        """Test playbook embedded string in string array"""
        tcex.playbook.create_string_array(variable, embedded_value)
        assert tcex.playbook.read(variable) == resolved_value

        # print('redis    : {}'.format(tcex.playbook.read(variable)))
        # print('resolved : {}'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(variable))))
        # print('resolved (type): {}'.format(type(resolved_value)))

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,embedded_value,resolved_value',
        [
            (
                '#App:0001:embedded.keyvalue.1!KeyValue',
                {'key': 'one', 'value': '#App:0001:string.3!String'},
                {'key': 'one', 'value': 'two'},
            )
        ],
    )
    def test_embedded_string_in_keyvalue(self, variable, embedded_value, resolved_value):
        """Test playbook embedded string array in string array"""
        tcex.playbook.create_key_value(variable, embedded_value)
        assert tcex.playbook.read(variable) == resolved_value

        # print('redis    : {}'.format(tcex.playbook.read(variable)))
        # print('resolved : {}'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(variable))))
        # print('resolved (type): {}'.format(type(resolved_value)))

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,embedded_value,resolved_value',
        [
            (
                '#App:0001:embedded.keyvalue.1!KeyValue',
                {'key': 'one', 'value': '#App:0001:array.1!StringArray'},
                {'key': 'one', 'value': ['two', 'three']},
            )
        ],
    )
    def test_embedded_string_array_in_keyvalue(self, variable, embedded_value, resolved_value):
        """Test playbook embedded string array in string array"""
        tcex.playbook.create_key_value(variable, embedded_value)
        assert tcex.playbook.read(variable) == resolved_value

        # print('redis    : {}'.format(tcex.playbook.read(variable)))
        # print('resolved : {}'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(variable))))
        # print('resolved (type): {}'.format(type(resolved_value)))

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,embedded_value,resolved_value',
        [
            (
                '#App:0001:embedded.keyvalue.1!KeyValue',
                {'key': 'one', 'value': '#App:0001:keyvalue.1!KeyValue'},
                {'key': 'one', 'value': {'value': 'two', 'key': 'kv1'}},
            )
        ],
    )
    def test_embedded_keyvalue_in_keyvalue(self, variable, embedded_value, resolved_value):
        """Test playbook embedded string array in string array"""
        tcex.playbook.create_key_value(variable, embedded_value)
        assert tcex.playbook.read(variable) == resolved_value

        # print('redis    : {}'.format(tcex.playbook.read(variable)))
        # print('resolved : {}'.format(resolved_value))
        # print('redis (type)   : {}'.format(type(tcex.playbook.read(variable))))
        # print('resolved (type): {}'.format(type(resolved_value)))

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None



# -*- coding: utf-8 -*-
"""ThreatConnect TI Victim"""

from tcex.tcex_ti.mappings.tcex_ti_mappings import TIMappings

# import local modules for dynamic reference
module = __import__(__name__)


class Victim(TIMappings):
    """Unique API calls for Victim API Endpoints"""

    def __init__(self, tcex, name, owner=None, **kwargs):
        super(Victim, self).__init__(tcex, 'Victim', 'victims', None, 'victim', None, owner)
        self._data['name'] = name

        for arg, value in kwargs.items():
            self.add_key_value(arg, value)

    @staticmethod
    def is_victim():
        return True

    def can_create(self):
        """
         If the name has been provided returns that the Victim can be created,
          otherwise returns that the Victim cannot be created.

         Returns:

         """
        if self._data.get('name'):
            return True
        return False

    def add_key_value(self, key, value):
        """
        Converts the value and adds it as a data field.

        Args:
            key:
            value:
        """
        if key == 'unique_id':
            self._unique_id = str(value)
        else:
            self._data[key] = value

    def _set_unique_id(self, json_response):
        """
        Sets the unique_id provided a json response.

        Args:
            json_response:
        """
        self.unique_id = json_response.get('id', '')

    @property
    def name(self):
        """Return Victim name."""
        return self._data.get('name')

    def add_asset(self, asset, asset_name, asset_type):
        """
        Adds a asset to the Victim

        Valid asset_type:
        + PHONE
        + EMAIL
        + NETWORK
        + SOCIAL
        + WEB

        Args:
            asset:
            asset_name:
            asset_type: PHONE, EMAIL, NETWORK, SOCIAL, or WEB

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if asset == 'PHONE':
            return self.tc_requests.add_victim_phone_asset(self.unique_id, asset_name)
        if asset == 'EMAIL':
            return self.tc_requests.add_victim_email_asset(self.unique_id, asset_name, asset_type)
        if asset == 'NETWORK':
            return self.tc_requests.add_victim_network_asset(self.unique_id, asset_name, asset_type)
        if asset == 'SOCIAL':
            return self.tc_requests.add_victim_social_asset(self.unique_id, asset_name, asset_type)
        if asset == 'WEB':
            return self.tc_requests.add_victim_web_asset(self.unique_id, asset_name)
        self._tcex.handle_error(
            925, ['asset_type', 'add_asset', 'asset_type', 'asset_type', asset_type]
        )
        return None

    def update_asset(self, asset, asset_id, asset_name, asset_type):
        """
        Update a asset of a Victim

        Valid asset_type:
        + PHONE
        + EMAIL
        + NETWORK
        + SOCIAL
        + WEB

        Args:
            asset:
            asset_name:
            asset_id:
            asset_type: PHONE, EMAIL, NETWORK, SOCIAL, or WEB

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if asset == 'PHONE':
            return self.tc_requests.update_victim_phone_asset(self.unique_id, asset_id, asset_name)
        if asset == 'EMAIL':
            return self.tc_requests.update_victim_email_asset(
                self.unique_id, asset_id, asset_name, asset_type
            )
        if asset == 'NETWORK':
            return self.tc_requests.update_victim_network_asset(
                self.unique_id, asset_id, asset_name, asset_type
            )
        if asset == 'SOCIAL':
            return self.tc_requests.update_victim_social_asset(
                self.unique_id, asset_id, asset_name, asset_type
            )
        if asset == 'WEB':
            return self.tc_requests.update_victim_web_asset(self.unique_id, asset_id, asset_name)

        self._tcex.handle_error(
            925, ['asset_type', 'update_asset', 'asset_type', 'asset_type', asset_type]
        )
        return None

    def asset(self, asset_id, asset_type, action='GET'):
        """
        Gets a asset of a Victim

        Valid asset_type:
        + PHONE
        + EMAIL
        + NETWORK
        + SOCIAL
        + WEB

        Args:
            asset_type:
            asset_id:
            action:

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if asset_type == 'PHONE':
            return self.tc_requests.victim_phone_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        if asset_type == 'EMAIL':
            return self.tc_requests.victim_email_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        if asset_type == 'NETWORK':
            return self.tc_requests.victim_network_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        if asset_type == 'SOCIAL':
            return self.tc_requests.victim_social_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        if asset_type == 'WEB':
            return self.tc_requests.victim_web_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        self._tcex.handle_error(
            925, ['asset_type', 'asset', 'asset_type', 'asset_type', asset_type]
        )
        return None

    def assets(self, asset_type=None):
        """
        Gets the assets of a Victim

        Args:
            asset_type:

        Yields: asset json

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if not asset_type:
            for a in self.tc_requests.victim_assets(self.api_type, self.api_branch, self.unique_id):
                yield a
        if asset_type == 'PHONE':
            for a in self.tc_requests.victim_phone_assets(
                self.api_type, self.api_branch, self.unique_id
            ):
                yield a
        if asset_type == 'EMAIL':
            for a in self.tc_requests.victim_email_assets(
                self.api_type, self.api_branch, self.unique_id
            ):
                yield a
        if asset_type == 'NETWORK':
            for a in self.tc_requests.victim_network_assets(
                self.api_type, self.api_branch, self.unique_id
            ):
                yield a
        if asset_type == 'SOCIAL':
            for a in self.tc_requests.victim_social_assets(
                self.api_type, self.api_branch, self.unique_id
            ):
                yield a
        if asset_type == 'WEB':
            for a in self.tc_requests.victim_web_assets(
                self.api_type, self.api_branch, self.unique_id
            ):
                yield a

        self._tcex.handle_error(
            925, ['asset_type', 'assets', 'asset_type', 'asset_type', asset_type]
        )

    def get_asset(self, asset_id, asset_type):
        """
        Gets a asset
        Args:
            asset_id:
            asset_type:

        Returns:

        """
        return self.asset(asset_id, asset_type=asset_type)

    def delete_asset(self, asset_id, asset_type):
        """
        Deletes a asset

        Args:
            asset_id:
            asset_type:
        """
        return self.asset(asset_id, asset_type=asset_type, action='DELETE')

    def phone_assets(self):
        """ Gets Phone assets """
        for pa in self.assets(asset_type='PHONE'):
            yield pa

    def email_assets(self):
        """ Gets Phone assets """
        for ea in self.assets(asset_type='EMAIL'):
            yield ea

    def network_assets(self):
        """ Gets Network assets """
        for na in self.assets(asset_type='NETWORK'):
            yield na

    def social_assets(self):
        """ Gets Social assets """
        for sa in self.assets(asset_type='SOCIAL'):
            yield sa

    def web_assets(self):
        """ Gets a Web assets """
        for wa in self.assets(asset_type='WEB'):
            yield wa

    def phone_asset(self, asset_id, action='GET'):
        """ Gets a Phone asset """
        return self.asset(asset_id, 'PHONE', action=action)

    def email_asset(self, asset_id, action='GET'):
        """ Gets a Email asset """
        return self.asset(asset_id, 'EMAIL', action=action)

    def network_asset(self, asset_id, action='GET'):
        """
        Gets a Network Asset
        Args:
            asset_id:
            action:

        Returns:

        """
        return self.asset(asset_id, 'NETWORK', action=action)

    def social_asset(self, asset_id, action='GET'):
        """
        Gets a Social Asset
        Args:
            asset_id:
            action:

        Returns:

        """
        return self.asset(asset_id, 'SOCIAL', action=action)

    def web_asset(self, asset_id, action='GET'):
        """
        Gets a Web Asset
        Args:
            asset_id:
            action:

        Returns:

        """
        return self.asset(asset_id, 'WEB', action=action)

    def get_phone_asset(self, asset_id):
        """
        Gets a Phone Asset
        Args:
            asset_id:
            action:

        Returns:

        """
        return self.get_asset(asset_id, 'PHONE')

    def get_email_asset(self, asset_id):
        """
        Gets a Email Asset
        Args:
            asset_id:
            action:

        Returns:

        """
        return self.get_asset(asset_id, 'EMAIL')

    def get_network_asset(self, asset_id):
        """
        Gets a Network Asset
        Args:
            asset_id:

        Returns:

        """
        return self.get_asset(asset_id, 'NETWORK')

    def get_social_asset(self, asset_id):
        """
        Gets a Social Asset
        Args:
            asset_id:

        Returns:

        """
        return self.get_asset(asset_id, 'SOCIAL')

    def get_web_asset(self, asset_id):
        """
        Gets a Web Asset
        Args:
            asset_id:

        Returns:

        """
        return self.get_asset(asset_id, 'WEB')

    def delete_phone_asset(self, asset_id):
        """
        Gets a Phone Asset
        Args:
            asset_id:

        Returns:

        """
        return self.delete_asset(asset_id, 'PHONE')

    def delete_email_asset(self, asset_id):
        """
        Deletes a Email Asset
        Args:
            asset_id:

        Returns:

        """
        return self.delete_asset(asset_id, 'EMAIL')

    def delete_network_asset(self, asset_id):
        """
        Deletes a Network Asset
        Args:
            asset_id:

        Returns:

        """
        return self.delete_asset(asset_id, 'NETWORK')

    def delete_social_asset(self, asset_id):
        """
        Deletes a Social Asset
        Args:
            asset_id:

        Returns:

        """
        return self.delete_asset(asset_id, 'SOCIAL')

    def delete_web_asset(self, asset_id):
        """
        Deletes a Web Asset
        Args:
            asset_id:

        Returns:

        """
        return self.delete_asset(asset_id, 'WEB')

    def add_phone_asset(self, name):
        """
        Adds a Phone Asset
        Args:
            name: The name provided to the phone asset

        Returns:

        """
        return self.add_asset('PHONE', name, None)

    def add_email_asset(self, name, asset_type):
        """
        Adds a Email Asset
        Args:
            name: The name provided to the email asset
            asset_type: The type provided to the email asset

        Returns:

        """
        self.add_asset('EMAIL', name, asset_type)

    def add_network_asset(self, name, asset_type):
        """
        Adds a Network Asset
        Args:
            name: The name provided to the network asset
            asset_type: The type provided to the network asset

        Returns:

        """
        self.add_asset('NETWORK', name, asset_type)

    def add_social_asset(self, name, asset_type):
        """
        Adds a Social Asset
        Args:
            name: The name provided to the social asset
            asset_type: The type provided to the social asset

        Returns:

        """
        self.add_asset('SOCIAL', name, asset_type)

    def add_web_asset(self, name):
        """
        Adds a Web Asset
        Args:
            name: The name provided to the web asset

        Returns:

        """
        self.add_asset('WEB', name, None)

    def update_phone_asset(self, asset_id, name):
        """
        Updates a Phone Asset
        Args:
            name: The name provided to the phone asset
            asset_id:

        Returns:

        """
        return self.update_asset('PHONE', asset_id, name, None)

    def update_email_asset(self, asset_id, name, asset_type):
        """
        Updates a Email Asset
        Args:
            name: The name provided to the email asset
            asset_type: The type provided to the email asset
            asset_id:

        Returns:

        """
        self.update_asset('EMAIL', asset_id, name, asset_type)

    def update_network_asset(self, asset_id, name, asset_type):
        """
        Updates a Network Asset
        Args:
            name: The name provided to the network asset
            asset_type: The type provided to the network asset
            asset_id:

        Returns:

        """
        self.update_asset('NETWORK', asset_id, name, asset_type)

    def update_social_asset(self, asset_id, name, asset_type):
        """
        Updates a Social Asset
        Args:
            name: The name provided to the social asset
            asset_type: The type provided to the social asset
            asset_id:

        Returns:

        """
        self.update_asset('SOCIAL', asset_id, name, asset_type)

    def update_web_asset(self, asset_id, name):
        """
        Updates a Web Asset
        Args:
            name: The name provided to the web asset
            asset_id:

        Returns:

        """
        self.update_asset('WEB', asset_id, name, None)



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""
# import time

# import pytest

# from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestUtils:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""
        # self.tcex = test()

    # def test_token_renewal(self):
    #     """Test token renewal"""
    #     expiration_time = int(time.time())
    #     owner_resource._request.session.auth._token_expiration = int(time.time())



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestAdversaryGroups:
    """Test TcEx Host Groups."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti

    def test_signature_get(
        self,
        signature_id=None,
        name='signature-name-42353',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature get."""
        # create
        if not signature_id:
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        # get
        ti = self.ti.signature(
            name,
            file_name,
            file_type,
            file_content,
            owner=tcex.args.tc_owner,
            unique_id=signature_id,
        )
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('name') == name

        # delete
        self.signature_delete(signature_id)

    def test_signature_get_attributes(
        self,
        signature_id=None,
        name='signature-name-12453',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature get."""
        # create
        if not signature_id:
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        self.test_signature_add_attribute(
            signature_id=signature_id, attribute_type='Description', attribute_value='test1'
        )
        self.test_signature_add_attribute(
            signature_id=signature_id, attribute_type='Description', attribute_value='test2'
        )
        self.test_signature_add_attribute(
            signature_id=signature_id, attribute_type='Description', attribute_value='test3'
        )

        # get attributes
        ti = self.ti.signature(name, file_name, file_type, file_content, unique_id=signature_id)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

        # delete
        self.signature_delete(signature_id)

    def test_signature_get_tags(
        self,
        signature_id=None,
        name='signature-name-64235',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature get."""
        # create
        if not signature_id:
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        self.test_signature_add_tag(signature_id=signature_id, tag='One')
        self.test_signature_add_tag(signature_id=signature_id, tag='Two')

        # get tags
        ti = self.ti.signature(name, file_name, file_type, file_content, unique_id=signature_id)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

        # delete
        self.signature_delete(signature_id)

    def test_signature_get_include(
        self,
        signature_id=None,
        name='signature-name-78159',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature get."""
        if not signature_id:
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        self.test_signature_add_attribute(
            signature_id=signature_id, attribute_type='Description', attribute_value='test123'
        )
        self.test_signature_add_label(signature_id=signature_id, label='TLP:RED')
        self.test_signature_add_tag(signature_id=signature_id, tag='PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.signature(name, file_name, file_type, file_content, unique_id=signature_id)
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('signature').get('name') == name
        assert ti_data.get('data').get('signature').get('attribute')[0].get('value') == 'test123'
        assert ti_data.get('data').get('signature').get('securityLabel')[0].get('name') == 'TLP:RED'
        assert ti_data.get('data').get('signature').get('tag')[0].get('name') == 'PyTest'

        # delete
        self.signature_delete(signature_id)

    def signature_create(
        self,
        name='signature-name-65341',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature create."""
        ti = self.ti.signature(name, file_name, file_type, file_content, owner=tcex.args.tc_owner)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('signature').get('name') == name
        assert ti_data.get('data').get('signature').get('fileName') == file_name
        assert ti_data.get('data').get('signature').get('fileType') == file_type
        return ti.unique_id

    def test_signature_add_attribute(
        self,
        signature_id=None,
        name='signature-name-nkjvb',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test signature attribute add."""

        should_delete = False
        if not signature_id:
            should_delete = True
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        ti = self.ti.signature(
            name,
            file_name,
            file_type,
            file_content,
            owner=tcex.args.tc_owner,
            unique_id=signature_id,
        )
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value
        if should_delete:
            self.signature_delete(signature_id)

    def test_signature_add_label(
        self,
        signature_id=None,
        name='signature-name-ds4vb',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
        label='TLP:GREEN',
    ):
        """Test signature attribute add."""
        should_delete = False
        if not signature_id:
            should_delete = True
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        ti = self.ti.signature(
            name,
            file_name,
            file_type,
            file_content,
            owner=tcex.args.tc_owner,
            unique_id=signature_id,
        )
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'
        if should_delete:
            self.signature_delete(signature_id)

    def test_signature_add_tag(
        self,
        signature_id=None,
        name='signature-name-fdsv23',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
        tag='Crimeware',
    ):
        """Test signature attribute add."""
        should_delete = False
        if not signature_id:
            should_delete = True
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        ti = self.ti.signature(
            name,
            file_name,
            file_type,
            file_content,
            owner=tcex.args.tc_owner,
            unique_id=signature_id,
        )
        r = ti.add_tag(tag)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'
        if should_delete:
            self.signature_delete(signature_id)

    def signature_delete(
        self,
        signature_id=None,
        name='signature-name-bdsfd',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature delete."""
        # create indicator
        if not signature_id:
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        # delete indicator
        ti = self.ti.signature(
            name,
            file_name,
            file_type,
            file_content,
            owner=tcex.args.tc_owner,
            unique_id=signature_id,
        )
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_signature_update(
        self,
        signature_id=None,
        name='signature-name-b3da3',
        file_name='signature-file-name-fdasr',
        file_type='Snort',
        file_content='signature-file-content-t5r32',
    ):
        """Test signature update."""
        # create indicator
        if not signature_id:
            signature_id = self.signature_create(name, file_name, file_type, file_content)

        name = 'signature-new-name-fdasb3'

        # update indicator
        ti = self.ti.signature(
            name,
            'signature-file-name.txt',
            'Bro',
            file_content,
            owner=tcex.args.tc_owner,
            unique_id=signature_id,
        )
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('signature').get('name') == name
        assert ti_data.get('data').get('signature').get('fileName') == 'signature-file-name.txt'
        assert ti_data.get('data').get('signature').get('fileType') == 'Bro'
        r = ti.download()
        assert r.status_code == 200

        # delete indicator
        self.signature_delete(signature_id)



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

import pytest

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestUtils:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""
        out_variables = [
            '#App:0001:one!String',
            '#App:0001:two!String',
            '#App:0001:three!String',
            '#App:0001:four!String',
            '#App:0001:a!StringArray',
        ]
        setattr(tcex.args, 'tc_playbook_out_variables', ','.join(out_variables))

    @pytest.mark.parametrize(
        'variable,value',
        [
            ('#App:0001:one!String', '1'),
            ('#App:0001:two!String', '2'),
            ('#App:0001:three!String', '3'),
            ('#App:0001:four!String', '4'),
            ('#App:0001:a!StringArray', ['a', 'b', 'c']),
        ],
    )
    def test_create_output(self, variable, value):
        """Test playbook create output"""
        parsed_variable = tcex.playbook.parse_variable(variable)
        variable_name = parsed_variable.get('name')
        variable_type = parsed_variable.get('type')
        tcex.playbook.create_output(variable_name, value, variable_type)
        assert tcex.playbook.read(variable) == value

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,value',
        [
            ('#App:0002:five!String', '1'),
            ('#App:0002:six!String', '2'),
            ('#App:0002:seven!String', '3'),
            ('#App:0002:eight!String', '4'),
        ],
    )
    def test_string(self, variable, value):
        """Test playbook create output"""
        tcex.playbook.create_string(variable, value)
        assert tcex.playbook.read_string(variable) == value

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None

    @pytest.mark.parametrize(
        'variable,value',
        [
            ('#App:0003:nine!StringArray', ['1', '1']),
            ('#App:0003:ten!StringArray', ['2', '2']),
            ('#App:0003:eleven!StringArray', ['3', '3']),
            ('#App:0003:twelve!StringArray', ['4', '4']),
        ],
    )
    def test_string_array(self, variable, value):
        """Test playbook create output"""
        tcex.playbook.create_string_array(variable, value)
        assert tcex.playbook.read_string_array(variable) == value

        tcex.playbook.delete(variable)
        assert tcex.playbook.read(variable) is None



# -*- coding: utf-8 -*-
"""ThreatConnect TI Host"""
from tcex.tcex_ti.mappings.indicator.tcex_ti_indicator import Indicator

try:
    from urllib import quote_plus  # Python 2
except ImportError:
    from urllib.parse import quote_plus  # Python


class Host(Indicator):
    """Unique API calls for Host API Endpoints"""

    def __init__(self, tcex, hostname, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            hostname (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            dns_active (bool, kwargs): If True DNS active is enabled for this indicator.
            whois_active (bool, kwargs): If True WhoIs active is enabled for this indicator.
        """
        super(Host, self).__init__(tcex, 'Host', 'host', 'hosts', owner, **kwargs)
        self.unique_id = kwargs.get('unique_id', hostname)
        self._data['hostName'] = hostname or self.unique_id
        if self.unique_id:
            self.unique_id = quote_plus(self.fully_decode_uri(self.unique_id))

    def can_create(self):
        """
        If the hostName has been provided returns that the File can be created, otherwise
        returns that the Host cannot be created.

        Returns:

        """
        if self.data.get('hostName'):
            return True
        return False

    def _set_unique_id(self, json_response):
        """
        Sets the unique_id provided a json response.

        Args:
            json_response:
        """
        self.unique_id = json_response.get('hostName', '')

    def dns_resolution(self):
        """
        Updates the Host DNS resolution

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.dns_resolution(
            self.api_type, self.api_branch, self.unique_id, owner=self.owner
        )



# -*- coding: utf-8 -*-
"""Test the TcEx Batch Module."""

from ..tcex_init import tcex


# pylint: disable=R0201,W0201
class TestLogs:
    """Test the TcEx Batch Module."""

    def setup_class(self):
        """Configure setup before all tests."""
        # self.tcex = test()

    def test_any_to_datetime(self):
        """Test any to datetime"""
        tcex.log.trace('TRACE LOGGING')
        tcex.log.debug('DEBUG LOGGING')
        tcex.log.info('INFO LOGGING')
        tcex.log.warning('WARNING LOGGING')
        tcex.log.error('ERROR LOGGING')



# -*- coding: utf-8 -*-
"""ThreatConnect TI URL"""
from tcex.tcex_ti.mappings.indicator.tcex_ti_indicator import Indicator

try:
    from urllib import quote_plus  # Python 2
except ImportError:
    from urllib.parse import quote_plus  # Python


class URL(Indicator):
    """Unique API calls for URL API Endpoints"""

    def __init__(self, tcex, text, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            text (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
            xid (str, kwargs): The external id for this Indicator.
        """
        super(URL, self).__init__(tcex, 'URL', 'url', 'urls', owner, **kwargs)
        self.unique_id = kwargs.get('unique_id', text)
        self.data['text'] = text or self.unique_id
        if self.unique_id:
            self.unique_id = quote_plus(self.fully_decode_uri(self.unique_id))

    def can_create(self):
        """
         If the text has been provided returns that the URL can be created, otherwise
         returns that the URL cannot be created.

         Returns:

         """
        if self.data.get('text'):
            return True
        return False

    def _set_unique_id(self, json_response):
        """
        Sets the unique_id provided a json response.

        Args:
            json_response:
        """
        self.unique_id = quote_plus(self.fully_decode_uri(json_response.get('text', '')))



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestFileIndicators:
    """Test TcEx Address Indicators."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti

    def teardown_class(self):
        """Configure setup before all tests."""
        hash_prefixes = ['A4', 'A5', 'A6', 'A7', 'A8', 'A9', 'B4', 'B5', 'B6', 'B7']
        for hp in hash_prefixes:
            unique_id = hp * 16
            ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=unique_id)
            ti.delete()

    def test_file_get(self, file_uuid=None, md5='A4', sha1='A4', sha256='A4'):
        """Test file get."""
        # create
        if not file_uuid:
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)

        # get
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('file').get('md5', None) == md5 * 16
        assert ti_data.get('data').get('file').get('sha1', None) == sha1 * 20
        assert ti_data.get('data').get('file').get('sha256', None) == sha256 * 32

        # delete
        self.test_file_delete(file_uuid)

    def test_file_get_attributes(self, file_uuid=None, md5='A5', sha1='A5', sha256='A5'):
        """Test file get."""
        # create
        if not file_uuid:
            file_uuid = self.file_create(md5 * 16, sha1 * 20, sha256 * 32)

        self.test_file_add_attribute(file_uuid, 'Description', 'test1')
        self.test_file_add_attribute(file_uuid, 'Description', 'test2')
        self.test_file_add_attribute(file_uuid, 'Description', 'test3')

        # get attributes
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

        # delete
        self.test_file_delete(file_uuid)

    def test_file_get_tags(self, file_uuid=None, md5='A6', sha1='A6', sha256='A6'):
        """Test file get."""
        # create
        if not file_uuid:
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)

        self.test_file_add_tag(file_uuid, 'One')
        self.test_file_add_tag(file_uuid, 'Two')

        # get tags
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

        # delete
        self.test_file_delete(file_uuid)

    def test_file_get_include(self, file_uuid=None, md5='A7', sha1='A7', sha256='A7'):
        """Test file get."""
        if not file_uuid:
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)

        self.test_file_add_attribute(
            file_uuid, attribute_type='Description', attribute_value='test123'
        )
        self.test_file_add_label(file_uuid, label='TLP:RED')
        self.test_file_add_tag(file_uuid, name='PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('file').get('attribute')[0].get('value') == 'test123'
        assert ti_data.get('data').get('file').get('securityLabel')[0].get('name') == 'TLP:RED'
        assert ti_data.get('data').get('file').get('tag')[0].get('name') == 'PyTest'

        # delete
        self.test_file_delete(file_uuid)

    def file_create(self, md5, sha1, sha256):
        """Test file create."""
        ti = self.ti.file(owner=tcex.args.tc_owner, md5=md5, sha1=sha1, sha256=sha256)
        r = ti.create()
        assert r.status_code == 201
        ti_data = r.json()
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('file').get('md5', None) == md5
        assert ti_data.get('data').get('file').get('sha1', None) == sha1
        assert ti_data.get('data').get('file').get('sha256', None) == sha256

        if md5:
            return md5
        if sha1:
            return sha1
        if sha256:
            return sha256
        return None

    def test_file_add_attribute(
        self,
        file_uuid=None,
        md5='A9',
        sha1='A9',
        sha256='A9',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test file attribute add."""
        should_delete = False
        if not file_uuid:
            should_delete = True
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value
        if should_delete:
            self.test_file_delete(file_uuid)

    def test_file_add_label(
        self, file_uuid=None, md5='B4', sha1='B4', sha256='B4', label='TLP:GREEN'
    ):
        """Test file attribute add."""
        should_delete = False
        if not file_uuid:
            should_delete = True
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'
        if should_delete:
            self.test_file_delete(file_uuid)

    def test_file_add_tag(self, file_uuid=None, md5='B5', sha1='B5', sha256='B5', name='Crimeware'):
        """Test file attribute add."""
        should_delete = False
        if not file_uuid:
            should_delete = True
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.add_tag(name=name)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'

        if should_delete:
            self.test_file_delete(file_uuid)

    def test_file_delete(self, file_uuid=None, md5='B6', sha1='B6', sha256='B6'):
        """Test file delete."""
        # create indicator
        if not file_uuid:
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_file_update(self, file_uuid=None, md5='B7', sha1='B7', sha256='B7'):
        """Test file update."""
        # create indicator
        if not file_uuid:
            file_uuid = self.file_create(md5=md5 * 16, sha1=sha1 * 20, sha256=sha256 * 32)
        ti = self.ti.file(owner=tcex.args.tc_owner, unique_id=file_uuid)
        r = ti.rating(5)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('file').get('rating') == 5.0

        r = ti.confidence(10)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('file').get('confidence') == 10

        # delete indicator
        self.test_file_delete(file_uuid)



# -*- coding: utf-8 -*-
"""Test the TcEx Notification Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestNotification:
    """Test the TcEx Notification Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @staticmethod
    def test_notification_organization():
        """Test org notification."""
        notification = tcex.notification()
        notification.org(notification_type='PyTest notification', priority='Low')
        status = notification.send(message='High alert send to organization.')
        assert status.get('status') == 'Success'

    @staticmethod
    def test_notification_recipients():
        """Test org notification."""
        notification = tcex.notification()
        notification.recipients(
            notification_type='PyTest recipients notification',
            recipients='pytest@tci.ninja',
            priority='High',
        )
        status = notification.send(message='High alert send to recipients.')
        assert status.get('status') == 'Success'



# -*- coding: utf-8 -*-
"""ThreatConnect TI Campaign """
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Campaign(Group):
    """Unique API calls for Campaign API Endpoints"""

    def __init__(self, tcex, name, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            first_seen (str, kwargs): The first seen datetime expression for this Group.
        """
        super(Campaign, self).__init__(
            tcex, 'Campaign', 'campaign', 'campaigns', owner=owner, name=name, **kwargs
        )

    def first_seen(self, first_seen):
        """
        Updates the campaign with the new first_seen date.

        Args:
            first_seen: The first_seen date. Converted to %Y-%m-%dT%H:%M:%SZ date format

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        first_seen = self._utils.format_datetime(first_seen, date_format='%Y-%m-%dT%H:%M:%SZ')
        self._data['firstSeen'] = first_seen
        request = {'firstSeen': first_seen}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)



# -*- coding: utf-8 -*-
"""App Decorators Module."""
import datetime

# import time


class Benchmark(object):
    """Log benchmarking times.

    This decorator will log the time of execution (benchmark_time) to the app.log file. It can be
    helpful in troubleshooting performance issues with Apps.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        import time

        @Debug()
        def my_method():
            time.sleep(1)
    """

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def benchmark(app, *args, **kwargs):
            """Iterate over data, calling the decorated function for each value.

            Args:
                app (class): The instance of the App class "self".
            """

            # before = float('{0:.4f}'.format(time.clock()))
            before = datetime.datetime.now()
            data = fn(app, *args, **kwargs)
            # after = float('{0:.4f}'.format(time.clock()))
            after = datetime.datetime.now()
            app.tcex.log.debug(
                'function: "{}", benchmark_time: "{}"'.format(
                    self.__class__.__name__, after - before
                )
            )
            return data

        return benchmark


class Debug(object):
    """Debug function inputs.

    This decorator will log the inputs to the function to assist in debugging an App.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @Debug()
        def my_method(arg1, args2):
            print(arg1, arg2)
    """

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def debug(app, *args, **kwargs):
            """Iterate over data, calling the decorated function for each value.

            Args:
                app (class): The instance of the App class "self".
            """

            data = fn(app, *args, **kwargs)
            app.tcex.log.debug(
                'function: "{}", args: "{}", kwargs: "{}"'.format(
                    self.__class__.__name__, vars(args), kwargs
                )
            )
            return data

        return debug


class FailOnInput(object):
    """Fail App if input value conditions are met.

    This decorator allows for the App to exit on conditions defined in the function
    parameters.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @FailOnInput(enable=True, values=[None, ''], msg='Invalid input provided.', arg=None)
        def my_method(data):
            return data.lowercase()
    """

    def __init__(self, enable, values, msg, arg=None):
        """Initialize Class Properties.

        Args:
            enable (boolean|str): Accepts a boolean or string value.  If a boolean value is
                provided that value will control enabling/disabling this feature. A string
                value should reference an item in the args namespace which resolves to a boolean.
                The value of this boolean will control enabling/disabling this feature.
            values (list): The values that, if matched, would trigger an exit.
            msg (str): The message to send to exit method.
            arg (str, optional): Defaults to None. The args Namespace value to use for the
                condition. If None the first value passed to the function will be used.
        """
        self.arg = arg
        self.enable = enable
        self.msg = msg
        self.values = values

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def fail(app, *args, **kwargs):
            """Call the function and store or append return value.

            Args:
                app (class): The instance of the App class "self".
            """

            # self.enable (e.g., True or 'fail_on_false') enables/disables this feature
            if isinstance(self.enable, bool):
                enabled = self.enable
                app.tcex.log.debug('Fail on input is ({}).'.format(self.enable))
            else:
                enabled = getattr(app.args, self.enable)
                app.tcex.log.debug('Fail on input is ({}) for ({}).'.format(enabled, self.enable))
                if not isinstance(enabled, bool):
                    app.tcex.playbook.exit(
                        1, 'The enable value must be a boolean for fail on input.'
                    )

            if enabled is True:
                if self.arg is None:
                    # grab the first arg passed to function to use in condition
                    arg_name = 'input'
                    conditional_value = app.tcex.playbook.read(list(args)[0], embedded=False)
                else:
                    # grab the arg from the args names space to use in condition
                    arg_name = self.arg
                    conditional_value = app.tcex.playbook.read(
                        getattr(app.args, self.arg), embedded=False
                    )

                if conditional_value in self.values:
                    app.tcex.log.error(
                        'Invalid value ({}) provided for ({}).'.format(conditional_value, arg_name)
                    )
                    app.tcex.exit(1, self.msg)

            return fn(app, *args, **kwargs)

        return fail


class FailOnOutput(object):
    """Fail App if return value (output) value conditions are met.

    This decorator allows for the App to exit on conditions defined in the function
    parameters.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @FailOnOutput(
            arg='fail_on_false', values=['false'], msg='Operation returned a value of "false".'
        )
        def my_method(data):
            return data.lowercase()
    """

    def __init__(self, enable, values, msg):
        """Initialize Class Properties.

        Args:
            enable (boolean|str): Accepts a boolean or string value.  If a boolean value is
                provided that value will control enabling/disabling this feature. A string
                value should reference an item in the args namespace which resolves to a boolean.
                The value of this boolean will control enabling/disabling this feature.
            values (list): The values that, if matched, would trigger an exit.
            msg (str): The message to send to exit method.

        """
        self.enable = enable
        self.msg = msg
        self.values = values

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def fail(app, *args, **kwargs):
            """Call the function and store or append return value.

            Args:
                app (class): The instance of the App class "self".
            """
            data = fn(app, *args, **kwargs)
            # self.enable (e.g., True or 'fail_on_false') enables/disables this feature
            if isinstance(self.enable, bool):
                enabled = self.enable
                app.tcex.log.debug('Fail on output is ({}).'.format(self.enable))
            else:
                enabled = getattr(app.args, self.enable)
                app.tcex.log.debug('Fail on output is ({}) for ({}).'.format(enabled, self.enable))
                if not isinstance(enabled, bool):
                    app.tcex.playbook.exit(
                        1, 'The enable value must be a boolean for fail on output.'
                    )

            failed = False
            if enabled is True:
                if isinstance(data, list):
                    # validate each value in the list of results.
                    for d in data:
                        if d in self.values:
                            failed = True
                else:
                    if data in self.values:
                        failed = True

                if failed:
                    app.tcex.exit(1, self.msg)
            return data

        return fail


class IterateOnArg(object):
    """Iterate on values stored in ``self.args`` namespace.

    This decorator will iterate over all value of the supplied arg and return results. This feature
    is helpful when processing a single value (String) or array of values (StringArray). If the App
    was provided the arg ``self.args.colors`` with a value of ``['blue', 'green', 'magenta']``, then
    this decorator would call the function 3 times. Each call to the function would pass one value
    from the array. The return values are stored and returned all at once after the last value is
    processed.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        # the args value of "colors" refers to ``self.args.colors``.
        @iterateOnArgs(arg='colors')
        def my_method(value):
            return value
    """

    def __init__(self, arg, default=None, fail_on=None):
        """Initialize Class Properties.

        Args:
            arg (str): The arg name from the App which contains the input. This input can be
                a Binary, BinaryArray, KeyValue, KeyValueArray, String, StringArray, TCEntity, or
                TCEntityArray.
            default (str, optional): Defaults to None. Default value to pass to method if arg
                value is None. Only supported for String or StringArray.
            fail_on (list, optional): Defaults to None. Fail if data read from Redis is in list.
        """
        self.arg = arg
        self.default = default
        self.fail_on = fail_on

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def loop(app, *args, **kwargs):
            """Iterate over data, calling the decorated function for each value.

            Args:
                app (class): The instance of the App class "self".
            """

            # retrieve data from Redis if variable and always return and array.
            r = []
            arg_data = app.tcex.playbook.read(getattr(app.args, self.arg))
            arg_type = app.tcex.playbook.variable_type(getattr(app.args, self.arg))
            if not isinstance(arg_data, list):
                arg_data = [arg_data]

            if not arg_data:
                app.tcex.exit(1, 'No data retrieved for arg ({}).'.format(self.arg))

            for s in arg_data:
                if s is None and self.default is not None:
                    # set value passed to method to default if value is None.
                    s = self.default
                    app.tcex.log.debug(
                        'a null input was provided, using default value "{}" instead.'.format(s)
                    )

                if self.fail_on is not None:
                    if s in self.fail_on:
                        app.tcex.playbook.exit(
                            1,
                            'Arg value for IterateOnArg matched fail_on value ({}).'.format(
                                self.fail_on
                            ),
                        )

                # Add logging for debug/troubleshooting
                if (
                    arg_type not in ['Binary', 'BinaryArray']
                    and app.tcex.log.getEffectiveLevel() == 10
                ):
                    log_string = str(s)
                    if len(log_string) > 100:
                        log_string = '{} ...'.format(log_string[:100])
                    app.tcex.log.debug('input value: {}'.format(log_string))

                # update the first item in the tuple
                args_list = list(args)
                try:
                    args_list[0] = s
                except IndexError:
                    args_list.append(s)
                args = tuple(args_list)
                r.append(fn(app, *args, **kwargs))
            return r

        return loop


class OnException(object):
    """Set exit message on failed execution.

    This decorator will catch the generic "Exception" error, log the supplied error message, set
    the "exit_message", and exit the App with an exit code of 1.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @OnException(msg='Failed to process JSON data.')
        def my_method(json_data):
            json.dumps(json_data)
    """

    def __init__(self, msg=None):
        """Initialize Class Properties.

        Args:
            msg (str): The message to send to exit method.
        """
        self.msg = msg

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def exception(app, *args, **kwargs):
            """Call the function and handle any exception.

            Args:
                app (class): The instance of the App class "self".
            """

            try:
                return fn(app, *args, **kwargs)
            except Exception as e:
                app.tcex.log.error('method failure ({})'.format(e))
                app.tcex.exit(1, self.msg)

        return exception


class OnSuccess(object):
    """Set exit message on successful execution.

    This decorator will set the supplied msg as the App "exit_message". Typically and App would
    only have 1 exit message so this decorator would typically be used in App that used "actions".

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @OnSuccess(msg='Successfully processed JSON data.')
        def my_method(json_data):
            json.dumps(json_data)
    """

    def __init__(self, msg=None):
        """Initialize Class Properties.

        Args:
            msg (str): The message to send to exit method.
        """

        self.msg = msg

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def completion(app, *args, **kwargs):
            """Call the function and handle any exception.

            Args:
                app (class): The instance of the App class "self".
            """
            app.exit_message = self.msg
            return fn(app, *args, **kwargs)

        return completion


class Output(object):
    """Store the method return value in self.<attribute>.

    This decorator will write, append, or extend the methods return value to the App attribute
    provided in the ``attribute`` input. The ``attribute`` must first be defined in the
    ``__init__()`` method of the App before the decorator is used.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        def __init__(self, _tcex):
            super(App, self).__init__(_tcex)
            self.output_strings = []  # Output decorator writes here.

        @Output(attribute='output_strings')
        def my_method(data):
            return data.lowercase()
    """

    def __init__(self, attribute):
        """Initialize Class Properties.

        Args:
            attribute (str): The name of the App attribute to write data.
        """

        self.attribute = attribute

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def output(app, *args, **kwargs):
            """Call the function and store or append return value.

            Args:
                app (class): The instance of the App class "self".
            """
            data = fn(app, *args, **kwargs)
            attr = getattr(app, self.attribute)
            if isinstance(data, list) and isinstance(attr, list):
                getattr(app, self.attribute).extend(data)
            elif isinstance(attr, list):
                getattr(app, self.attribute).append(data)
            else:
                setattr(app, self.attribute, data)
            return data

        return output


class ReadArg(object):
    """Read value of App arg resolving any playbook variables.

    This decorator will read the args from Redis, if it is a playbook variable, and pass the
    resolved value to the function.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @ReadArg(arg='color')
        def my_method(color):
            print('color', color)
    """

    def __init__(self, arg, array=False, default=None, fail_on=None):
        """Initialize Class Properties.

        Args:
            arg (str): The arg name from the App which contains the input. This input can be
                a Binary, BinaryArray, KeyValue, KeyValueArray, String, StringArray, TCEntity, or
                TCEntityArray.
            array (bool, optional): Defaults to False. If True the arg value will always be
                returned as an array.
            default (str, optional): Defaults to None. Default value to pass to method if arg
                value is None. Only supported for String or StringArray.
            fail_on (list, optional): Defaults to None. Fail if data read from Redis is in list.
        """
        self.arg = arg
        self.array = array
        self.default = default
        self.fail_on = fail_on

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def read(app, *args, **kwargs):
            """Retrieve/Read data from Redis.

            Args:
                app (class): The instance of the App class "self".
            """
            # retrieve data from Redis and call decorated function
            args_list = list(args)
            try:
                arg_data = app.tcex.playbook.read(getattr(app.args, self.arg), self.array)
                arg_type = app.tcex.playbook.variable_type(getattr(app.args, self.arg))
                if self.default is not None and arg_data is None:
                    arg_data = self.default
                if self.fail_on is not None:
                    if arg_data in self.fail_on:
                        app.tcex.playbook.exit(
                            1,
                            'Arg value for ReadArg matched fail_on value ({}).'.format(
                                self.fail_on
                            ),
                        )
                args_list[0] = arg_data

                # Add logging for debug/troubleshooting
                if (
                    arg_type not in ['Binary', 'BinaryArray']
                    and app.tcex.log.getEffectiveLevel() == 10
                ):
                    log_string = str(arg_data)
                    if len(log_string) > 100:
                        log_string = '{} ...'.format(log_string[:100])
                    app.tcex.log.debug('input value: {}'.format(log_string))
            except IndexError:
                args_list.append(app.tcex.playbook.read(getattr(app.args, self.arg), self.array))
            args = tuple(args_list)

            return fn(app, *args, **kwargs)

        return read


class WriteOutput(object):
    """Write the App output variables to Redis.

    This decorator will take the functions return value and write the data to Redis using the
    key and variable_type. An optional hard coded value can be passed, which will override the
    return value. If multiple value are provided for the same output variable there is an option
    to overwrite the previous value.

    This decorator is intended for very simple Apps. Using the `write_output()` method of the App
    template is the recommended way of writing output data.

    .. code-block:: python
        :linenos:
        :lineno-start: 1

        @WriteOutput(key='color', variable_type='String')
        def my_method(color):
            return color.lowercase()
    """

    def __init__(self, key, variable_type, value=None, overwrite=True):
        """Initialize Class Properties.

        Args:
            key (str): The name of the playbook output variable.
            variable_type (str): The type for the playbook output variable.  Supported types are:
                String, Binary, KeyValue, TCEntity, TCEnhancedEntity, StringArray,
                BinaryArray, KeyValueArray, TCEntityArray, TCEnhancedEntityArray.
            value (str): When a value is provided that value will be written instead of the data
                returned from the function call.
            overwrite (bool): When True and more than one value is provided for the same variable
                the previous value will be overwritten.
        """
        self.key = key
        self.overwrite = overwrite
        self.value = value
        self.variable_type = variable_type

    def __call__(self, fn):
        """Implement __call__ function for decorator.

        Args:
            fn (function): The decorated function.

        Returns:
            function: The custom decorator function.
        """

        def output(app, *args, **kwargs):
            """Call the function and store or append return value.

            Args:
                app (class): The instance of the App class "self".
            """
            data = fn(app, *args, **kwargs)
            index = '{}-{}'.format(self.key, self.variable_type)
            if self.value is not None:
                # store user provided data
                app.tcex.playbook.add_output(self.key, self.value, self.variable_type)
            elif app.tcex.playbook.output_data.get(index) and not self.overwrite:
                # skip data since a previous value has already been written
                pass
            else:
                # store data returned by function call
                app.tcex.playbook.add_output(self.key, data, self.variable_type)
            return data

        return output



# -*- coding: utf-8 -*-
"""TcEx Runtime App Test Case"""
import os
from six import string_types
from .test_case import TestCase


class TestCaseJob(TestCase):
    """App TestCase Class"""

    _output_variables = None
    redis_client = None

    @staticmethod
    def create_shelf_dir(shelf_path):
        """Create a directory in log with the context name containing the batch data."""
        if not os.path.isdir(shelf_path):
            os.makedirs(shelf_path)
            with open(os.path.join(shelf_path, 'DEBUG'), 'a'):
                os.utime(os.path.join(shelf_path, 'DEBUG'), None)

    def run(self, args):  # pylint: disable=too-many-return-statements
        """Run the Playbook App.

        Args:
            args (dict): The App CLI args.

        Returns:
            [type]: [description]
        """
        # resolve env vars
        for k, v in list(args.items()):
            if isinstance(v, string_types):
                args[k] = self.resolve_env_args(v)

        self.log_data('run', 'args', args)
        app = self.app(args)

        # Start
        exit_code = self.run_app_method(app, 'start')
        if exit_code != 0:
            return exit_code

        # Run
        exit_code = self.run_app_method(app, 'run')
        if exit_code != 0:
            return exit_code

        # Write Output
        exit_code = self.run_app_method(app, 'write_output')
        if exit_code != 0:
            return exit_code

        # Done
        exit_code = self.run_app_method(app, 'done')
        if exit_code != 0:
            return exit_code

        return self._exit(app.tcex.exit_code)

    def run_profile(self, profile_name):
        """Run an App using the profile name."""
        profile = self.profile(profile_name)
        if not profile:
            self.log.error('No profile named {} found.'.format(profile_name))
            return self._exit(1)

        args = {'tc_temp_path': os.path.join(self._app_path, 'log', self.context)}
        self.create_shelf_dir(args['tc_temp_path'])

        # build args from install.json
        args.update(profile.get('inputs', {}).get('required', {}))
        args.update(profile.get('inputs', {}).get('optional', {}))
        if not args:
            self.log.error('No profile named {} found.'.format(profile_name))
            return self._exit(1)

        # run the App
        exit_code = self.run(args)

        return self._exit(exit_code)



# -*- coding: utf-8 -*-
"""ThreatConnect TI Adversary """
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Adversary(Group):
    """Unique API calls for Adversary API Endpoints"""

    def __init__(self, tcex, name, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
        """
        super(Adversary, self).__init__(
            tcex, 'Adversary', 'adversary', 'adversaries', owner=owner, name=name, **kwargs
        )

    def add_asset(self, asset_type, asset_name):
        """
        Add an asset to the adversary
        Args:
            asset_type: (str) Either PHONE, HANDLER, or URL
            asset_name: (str) the value for the asset

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if asset_type == 'PHONE':
            return self.tc_requests.add_adversary_phone_asset(
                self.api_type, self.api_branch, self.unique_id, asset_name
            )
        if asset_type == 'HANDLER':
            return self.tc_requests.add_adversary_handler_asset(
                self.api_type, self.api_branch, self.unique_id, asset_name
            )
        if asset_type == 'URL':
            return self.tc_requests.add_adversary_url_asset(
                self.api_type, self.api_branch, self.unique_id, asset_name
            )
        self._tcex.handle_error(
            925, ['asset_type', 'assets', 'asset_type', 'asset_type', asset_type]
        )
        return None

    def asset(self, asset_id, asset_type, action='GET'):
        """
        Gets the asset with the provided id
        Args:
            asset_id: The id of the asset to be retrieved
            asset_type: (str) Either PHONE, HANDLER, or URL
            action:

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if asset_type == 'PHONE':
            return self.tc_requests.adversary_phone_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        if asset_type == 'HANDLER':
            return self.tc_requests.adversary_handle_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        if asset_type == 'URL':
            return self.tc_requests.adversary_url_asset(
                self.api_type, self.api_branch, self.unique_id, asset_id, action=action
            )
        self._tcex.handle_error(
            925, ['asset_type', 'assets', 'asset_type', 'asset_type', asset_type]
        )
        return None

    def assets(self, asset_type=None):
        """
        Retrieves all of the assets of a given asset_type

        Args:
            asset_type: (str) Either None, PHONE, HANDLER, or URL

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        if not asset_type:
            return self.tc_requests.adversary_assets(self.api_type, self.api_branch, self.unique_id)
        if asset_type == 'PHONE':
            return self.tc_requests.adversary_phone_assets(
                self.api_type, self.api_branch, self.unique_id
            )
        if asset_type == 'HANDLER':
            return self.tc_requests.adversary_handle_assets(
                self.api_type, self.api_branch, self.unique_id
            )
        if asset_type == 'URL':
            return self.tc_requests.adversary_url_assets(
                self.api_type, self.api_branch, self.unique_id
            )

        self._tcex.handle_error(
            925, ['asset_type', 'assets', 'asset_type', 'asset_type', asset_type]
        )
        return None

    def get_asset(self, asset_id, asset_type):
        """
        Gets the asset with the provided asset_id & asset_type.

        Args:
            asset_id: The id of the asset.
            asset_type: The asset type.

        Returns:

        """
        return self.asset(asset_id, asset_type=asset_type)

    def delete_asset(self, asset_id, asset_type):
        """
        Delete the asset with the provided asset_id.

        Args:
            asset_id: The id of the asset.
            asset_type: The asset type.

        Returns:

        """
        return self.asset(asset_id, asset_type=asset_type, action='DELETE')

    def handle_assets(self):
        """

        Returns: all of the handle assets

        """
        return self.assets(asset_type='HANDLER')

    def phone_assets(self):
        """

        Returns: all of the phone assets

        """
        return self.assets(asset_type='PHONE')

    def url_assets(self):
        """

        Returns: all of the url assets

        """
        return self.assets(asset_type='URL')

    def handle_asset(self, asset_id, action='GET'):
        """
        Gets the phone asset with the passed in id.

        Args:
            asset_id: The id  of the asset to be retrieved
            action:

        Returns:

        """
        return self.asset(asset_id, 'HANDLER', action=action)

    def phone_asset(self, asset_id, action='GET'):
        """
        Gets the phone asset with the passed in id.

        Args:
            asset_id: The id  of the asset to be retrieved
            action:

        Returns:

        """
        return self.asset(asset_id, 'PHONE', action=action)

    def url_asset(self, asset_id, action='GET'):
        """
        Gets the url asset with the passed in id.

        Args:
            asset_id: The id  of the asset to be retrieved
            action:

        Returns:

        """
        return self.asset(asset_id, 'URL', action=action)

    def get_handle_asset(self, asset_id):
        """
        Gets the handle asset with the passed in id

        Args:
            asset_id: The id  of the asset to be retrieved

        Returns:

        """
        return self.get_asset(asset_id, 'HANDLER')

    def get_phone_asset(self, asset_id):
        """
        Gets the phone asset with the passed in id

        Args:
            asset_id: The id  of the asset to be retrieved

        Returns:

        """
        return self.get_asset(asset_id, 'PHONE')

    def get_url_asset(self, asset_id):
        """
        Gets the url asset with the passed in id

        Args:
            asset_id: The id  of the asset to be retrieved

        Returns:

        """
        return self.get_asset(asset_id, 'URL')

    def delete_phone_asset(self, asset_id):
        """
        Delete the phone asset with the passed in id

        Args:
            asset_id: The id of the asset to be deleted

        Returns:

        """
        return self.delete_asset(asset_id, 'PHONE')

    def delete_url_asset(self, asset_id):
        """
        Delete the url asset with the passed in id

        Args:
            asset_id: The id of the asset to be deleted

        Returns:

        """
        return self.delete_asset(asset_id, 'URL')

    def delete_handler_asset(self, asset_id):
        """
        Delete the handler asset with the passed in id

        Args:
            asset_id: The id of the asset to be deleted

        Returns:

        """
        return self.delete_asset(asset_id, 'HANDLER')

    def add_handler_asset(self, name):
        """
        Add a Handler asset to the adversary.

        Args:
            name: The name of the Handler asset

        Returns:

        """
        return self.add_asset('HANDLER', name)

    def add_phone_asset(self, name):
        """
        Add a phone asset to the adversary.

        Args:
            name: The name of the phone asset

        Returns:

        """
        self.add_asset('PHONE', name)

    def add_url_asset(self, name):
        """
        Add a URL asset to the adversary.

        Args:
            name: The name of the URL asset

        Returns:

        """
        self.add_asset('URL', name)



# -*- coding: utf-8 -*-
"""ThreatConnect TI Threat"""
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Threat(Group):
    """Unique API calls for Threat API Endpoints"""

    def __init__(self, tcex, name, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
        """
        super(Threat, self).__init__(
            tcex, 'Threat', 'threat', 'threats', owner=owner, name=name, **kwargs
        )



# -*- coding: utf-8 -*-
"""ThreatConnect TI User Agent"""
from tcex.tcex_ti.mappings.indicator.tcex_ti_indicator import Indicator


class UserAgent(Indicator):
    """Unique API calls for UserAgent API Endpoints"""

    def __init__(self, tcex, text, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            text (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
        """
        super(UserAgent, self).__init__(
            tcex, 'User Agent', 'userAgent', 'userAgents', owner, **kwargs
        )
        self.unique_id = kwargs.get('unique_id', text)
        self.data['User Agent String'] = text or self.unique_id

    def can_create(self):
        """
         If the text has been provided returns that the UserAgent can be created, otherwise
         returns that the UserAgent cannot be created.

         Returns:

         """
        if self.data.get('User Agent String'):
            return True
        return False

    def _set_unique_id(self, json_response):
        """
        Sets the unique_id provided a json response.

        Args:
            json_response:
        """
        self.unique_id = json_response.get('User Agent String', '')



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""
from ..tcex_init import tcex


# pylint: disable=W0201
# pylint: disable=E1101
class TestCIDRIndicators:
    """Test TcEx CIDR Indicators."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti
        blocks = [
            '1.1.1.1/8',
            '1.1.1.2/8',
            '1.1.1.3/8',
            '1.1.1.4/8',
            '1.1.1.5/8',
            '1.1.1.6/8',
            '1.1.1.7/8',
        ]
        for block in blocks:
            ti = self.ti.indicator(indicator_type='CIDR', owner=tcex.args.tc_owner, Block=block)
            ti.delete()

    def test_cidr_get(self, block='1.1.1.1/8'):
        """Test cidr get."""
        # create
        self.cidr_create(block)

        # get
        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('Block') == block

    def test_cidr_get_attributes(self, block='1.1.1.2/8'):
        """Test cidr get."""
        # create
        self.cidr_create(block)
        self.test_cidr_add_attribute(False, block, 'Description', 'test1')
        self.test_cidr_add_attribute(False, block, 'Description', 'test2')
        self.test_cidr_add_attribute(False, block, 'Description', 'test3')

        # get attributes
        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

    def test_cidr_get_tags(self, block='1.1.1.3/8'):
        """Test cidr get."""
        # create
        self.cidr_create(block)
        self.test_cidr_add_tag(False, block, 'One')
        self.test_cidr_add_tag(False, block, 'Two')

        # get tags
        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

    def test_cidr_get_include(self, block='1.1.1.8/8'):
        """Test cidr get."""
        self.cidr_create(block)

        self.test_cidr_add_attribute(False, block, 'Description', 'test123')
        self.test_cidr_add_label(False, block, 'TLP:RED')
        self.test_cidr_add_tag(False, block, 'PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        r = ti.single(params=parameters)
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('Block') == block
        assert ti_data.get('data').get(ti.api_entity).get('attribute')[0].get('value') == 'test123'
        assert (
            ti_data.get('data').get(ti.api_entity).get('securityLabel')[0].get('name') == 'TLP:RED'
        )
        assert ti_data.get('data').get(ti.api_entity).get('tag')[0].get('name') == 'PyTest'

    def cidr_create(self, block='1.1.1.7/8'):
        """Test cidr create."""
        ti = self.ti.indicator(indicator_type='CIDR', owner=tcex.args.tc_owner, Block=block)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('Block') == block

    def test_cidr_add_attribute(
        self,
        should_create=True,
        block='1.1.1.1/8',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test cidr attribute add."""
        if should_create:
            self.cidr_create(block)

        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value

    def test_cidr_add_label(self, should_create=True, block='1.1.1.6/8', label='TLP:GREEN'):
        """Test cidr attribute add."""
        if should_create:
            self.cidr_create(block)

        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'

    def test_cidr_add_tag(self, should_create=True, block='1.1.1.5/8', name='Crimeware'):
        """Test cidr attribute add."""
        if should_create:
            self.cidr_create(block)

        ti = self.ti.cidr(block, owner=tcex.args.tc_owner)
        r = ti.add_tag(name=name)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'

    def cidr_delete(self, block):
        """Test cidr delete."""
        # create indicator
        self.cidr_create(block)

        # delete indicator
        ti = self.ti.indicator(indicator_type='CIDR', owner=tcex.args.tc_owner, block=block)
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_cidr_update(self, block='1.1.1.4/8'):
        """Test cidr update."""
        # create indicator
        self.cidr_create(block)

        # update indicator
        ti = self.ti.indicator(
            indicator_type='CIDR', owner=tcex.args.tc_owner, Block=block, rating=5, confidence=10
        )
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('rating') == 5.0
        assert ti_data.get('data').get(ti.api_entity).get('confidence') == 10



# -*- coding: utf-8 -*-
"""ThreatConnect TI Event """
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Event(Group):
    """Unique API calls for Event API Endpoints"""

    def __init__(self, tcex, name, owner=None, **kwargs):
        """Initialize Class Properties.

        Valid status:
        + Escalated
        + False Positive
        + Needs Review
        + No Further Action

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            event_date (str, kwargs): The event datetime expression for this Group.
            status (str, kwargs): The status for this Group.
        """
        super(Event, self).__init__(
            tcex, 'Event', 'event', 'events', owner=owner, name=name, **kwargs
        )

    def status(self, status):
        """
        Updates the events status.

        Valid status:
        + Escalated
        + False Positive
        + Needs Review
        + No Further Action

        Args:
            status: Escalated, False Positive, Needs Review, No Further Action

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['status'] = status
        request = {'status': status}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)

    def event_date(self, event_date):
        """
        Updates the events event_date.

        Args:
            event_date: Converted to %Y-%m-%dT%H:%M:%SZ date format

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        event_date = self._utils.format_datetime(event_date, date_format='%Y-%m-%dT%H:%M:%SZ')
        self._data['eventDate'] = event_date
        request = {'eventDate': event_date}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)



# -*- coding: utf-8 -*-
"""ThreatConnect TI Adversary """
from tcex.tcex_ti.mappings.tcex_ti_mappings import TIMappings

try:
    from urllib import quote_plus  # Python 2
except ImportError:
    from urllib.parse import quote_plus  # Python


class Task(TIMappings):
    """Unique API calls for Tasks API Endpoints"""

    def __init__(
        self, tcex, name, status, due_date, reminder_date, escalation_date, owner=None, **kwargs
    ):
        """Initialize Class Properties.

        Valid status:
        + Not Started
        + In Progress
        + Completed
        + Waiting on Someone
        + Deferred

        Args:
            tcex:
            status (str): Not started, In Progress, Completed, Waiting on Someone, Deferred
            due_date: Converted to %Y-%m-%dT%H:%M:%SZ date format
            reminder_date: Converted to %Y-%m-%dT%H:%M:%SZ date format
            escalation_date: Converted to %Y-%m-%dT%H:%M:%SZ date format
            **kwargs:
            name (str): The name for this Group.
        """

        super(Task, self).__init__(tcex, 'Task', 'tasks', None, 'task', None, owner)
        self._data['name'] = name
        if status:
            self._data['status'] = status
        if due_date:
            self._data['dueDate'] = due_date
        if reminder_date:
            self._data['reminderDate'] = reminder_date
        if escalation_date:
            self._data['escalationDate'] = escalation_date

        for arg, value in kwargs.items():
            self.add_key_value(arg, value)

    @property
    def name(self):
        """Return Task name."""
        return self._data.get('name')

    @name.setter
    def name(self, name):
        """

        Args:
            name:
        """
        self._data['name'] = name

    @staticmethod
    def is_task():
        """
        Indicates that this is a task object
        Returns:

        """
        return True

    def _set_unique_id(self, json_response):
        """

        Args:
            json_response:
        """
        self.unique_id = json_response.get('id', '')

    def status(self, status):
        """
        Valid status:
        + Not Started
        + In Progress
        + Completed
        + Waiting on Someone
        + Deferred

        Args:
            status: Not Started, In Progress, Completed, Waiting on Someone, Deferred
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['status'] = status
        request = {'status': status}
        return self.tc_requests.update(self.api_type, self.api_sub_type, self.unique_id, request)

    def due_date(self, due_date):
        """
        Sets the task due_date
        Args:
            due_date: Converted to %Y-%m-%dT%H:%M:%SZ date format
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        due_date = self._utils.format_datetime(due_date, date_format='%Y-%m-%dT%H:%M:%SZ')
        self._data['dueDate'] = due_date
        request = {'dueDate': due_date}
        return self.tc_requests.update(self.api_type, self.api_sub_type, self.unique_id, request)

    def reminder_date(self, reminder_date):
        """
        Sets the task reminder_date
        Args:
            reminder_date: Converted to %Y-%m-%dT%H:%M:%SZ date format
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        reminder_date = self._utils.format_datetime(reminder_date, date_format='%Y-%m-%dT%H:%M:%SZ')
        self._data['reminderDate'] = reminder_date
        request = {'reminderDate': reminder_date}
        return self.tc_requests.update(self.api_type, self.api_sub_type, self.unique_id, request)

    def escalation_date(self, escalation_date):
        """
        Sets the task escalation_date
        Args:
            escalation_date: Converted to %Y-%m-%dT%H:%M:%SZ date format
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        escalation_date = self._utils.format_datetime(
            escalation_date, date_format='%Y-%m-%dT%H:%M:%SZ'
        )
        self._data['escalationDate'] = escalation_date
        request = {'escalationDate': escalation_date}
        return self.tc_requests.update(self.api_type, self.api_sub_type, self.unique_id, request)

    def assignees(self):
        """
        Gets the task assignees
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        for a in self.tc_requests.assignees(self.api_type, self.api_sub_type, self.unique_id):
            yield a

    def assignee(self, assignee_id, action='ADD'):
        """
        Adds a assignee to the task

        Args:
            assignee_id: The id of the assignee to be added
            action:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.assignee(
            self.api_type, self.api_sub_type, self.unique_id, assignee_id, action=action
        )

    def add_assignee(self, assignee_id):
        """
        Adds a assignee to the task

        Args:
            assignee_id: The id of the assignee to be added

        """
        return self.assignee(assignee_id)

    def get_assignee(self, assignee_id):
        """
        Gets a assignee from a task

        Args:
            assignee_id: The id of the assignee to be added

        """
        return self.assignee(assignee_id, action='GET')

    def delete_assignee(self, assignee_id):
        """
        Deletes a assignee from a task

        Args:
            assignee_id: The id of the assignee to be added

        """
        return self.assignee(assignee_id, action='DELETE')

    def escalatees(self):
        """
        Gets the task escalatees
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        for e in self.tc_requests.escalatees(self.api_type, self.api_sub_type, self.unique_id):
            yield e

    def escalatee(self, escalatee_id, action='ADD'):
        """
        Adds a assignee to the task

        Args:
            escalatee_id: The id of the escalatee to be added
            action:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.escalatee(
            self.api_type, self.api_sub_type, self.unique_id, escalatee_id, action=action
        )

    def add_escalatee(self, escalatee_id):
        """
        Adds a escalatee to the task

        Args:
            escalatee_id: The id of the escalatee to be added

        """
        return self.assignee(escalatee_id)

    def get_escalatee(self, escalatee_id):
        """
        Gets a escalatee from a task

        Args:
            escalatee_id: The id of the escalatee to be added

        """
        return self.assignee(escalatee_id, action='GET')

    def delete_escalatee(self, escalatee_id):
        """
        Deletes a escalatee from a task

        Args:
            escalatee_id: The id of the escalatee to be added

        """
        return self.assignee(escalatee_id, action='DELETE')

    def add_key_value(self, key, value):
        """
        Converts the value and adds it as a data field.

        Args:
            key:
            value:
        """
        if key == 'unique_id':
            self._unique_id = quote_plus(str(value))
        self._data[key] = value

    def can_create(self):
        """
         If the name has been provided returns that the Task can be created, otherwise
         returns that the Task cannot be created.

         Returns:

         """
        if not self.data.get('name', None):
            return False
        return True



# -*- coding: utf-8 -*-
"""ThreatConnect TI Registry Key"""
from tcex.tcex_ti.mappings.indicator.tcex_ti_indicator import Indicator


class RegistryKey(Indicator):
    """Unique API calls for RegistryKey API Endpoints"""

    def __init__(self, tcex, key_name, value_name, value_type, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            key_name (str): The key_name value for this Indicator.
            value_name (str): The value_name value for this Indicator.
            value_type (str): The value_type value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
        """
        super(RegistryKey, self).__init__(
            tcex, 'Registry Key', 'registryKey', 'registryKeys', owner, **kwargs
        )
        self.unique_id = kwargs.get('unique_id', key_name)
        self.data['Key Name'] = key_name or self.unique_id
        self.data['Value Name'] = value_name
        self.data['Value Type'] = value_type

    def can_create(self):
        """
         If the key_name, value_name, and value_type has been provided returns that the
         Registry Key can be created, otherwise returns that the Registry Key cannot be created.

         Returns:

         """
        if (
            self.data.get('Key Name')
            and self.data.get('Value Name')
            and self.data.get('Value Type')
        ):
            return True
        return False

    def _set_unique_id(self, json_response):
        """
        Sets the unique_id provided a json response.

        Args:
            json_response:
        """
        self.unique_id = json_response.get('Key Name', '')



# -*- coding: utf-8 -*-
"""ThreatConnect TI Address"""
from tcex.tcex_ti.mappings.indicator.tcex_ti_indicator import Indicator


class Address(Indicator):
    """Unique API calls for Address API Endpoints"""

    def __init__(self, tcex, ip, owner=None, **kwargs):
        """Initialize Class Properties.

        Args:
            ip (str): The value for this Indicator.
            active (bool, kwargs): If False the indicator is marked "inactive" in TC.
            confidence (str, kwargs): The threat confidence for this Indicator.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            last_modified (str, kwargs): The date timestamp the Indicator was last modified.
            private_flag (bool, kwargs): If True the indicator is marked as private in TC.
            rating (str, kwargs): The threat rating for this Indicator.
        """
        super(Address, self).__init__(tcex, 'Address', 'address', 'addresses', owner, **kwargs)
        self.unique_id = kwargs.get('unique_id', ip)
        self.data['ip'] = ip or self.unique_id

    def can_create(self):
        """
        If the ip address has been provided returns that the address can be created, otherwise
        returns that the address cannot be created.

        Returns:

        """
        if self.data.get('ip'):
            return True
        return False

    def _set_unique_id(self, json_response):
        """
        Sets the unique_id provided a json response.

        Args:
            json_response:
        """
        self.unique_id = json_response.get('ip', '')

    def dns_resolution(self):
        """
        Updates the DNS resolution.

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])
        return self.tc_requests.dns_resolution(
            self.api_type, self.api_branch, self.unique_id, owner=self.owner
        )



# -*- coding: utf-8 -*-
"""TcEx Framework Logger module"""
import logging
from logging.handlers import RotatingFileHandler
import os
import platform
import sys
import time
from inspect import getframeinfo, stack


# Create trace logging
logging.TRACE = logging.DEBUG - 5
logging.addLevelName(logging.TRACE, 'TRACE')


class TraceLogger(logging.Logger):
    """Add trace level to logging"""

    _in_trace = False

    def findCaller(self, stack_info=False):
        """Find the caller for the current log event."""
        caller = None
        depth = 3
        while True:
            # search for the correct calling method
            caller = getframeinfo(stack()[depth][0])
            if caller.function != 'trace' or depth >= 6:
                break
            depth += 1

        if sys.version_info < (3,):
            # return value for py2
            return (caller.filename, caller.lineno, caller.function)
        return (caller.filename, caller.lineno, caller.function, None)

    def trace(self, msg, *args, **kwargs):
        """Set trace logging level"""
        self.log(logging.TRACE, msg, *args, **kwargs)


class TcExLogger(object):
    """Framework logger module."""

    def __init__(self, tcex):
        """Initialize Class Properties.

        Args:
            tcex (tcex.TcEx): Instance of TcEx class.
        """
        self.tcex = tcex

    @property
    def _logger(self):
        """Return the logger. The default_args property is not available in init."""
        logging.setLoggerClass(TraceLogger)
        logger = logging.getLogger(self.tcex.default_args.tc_log_name)
        logger.setLevel(logging.TRACE)
        return logger

    # @property
    # def _formatter(self):
    #     """Return log formatter."""
    #     tx_format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s '
    #     tx_format += '(%(filename)s:%(funcName)s:%(lineno)d)'
    #     return logging.Formatter(tx_format)

    @property
    def log(self):
        """Return logger."""
        return self._logger

    @property
    def logging_level(self):
        """Return the configured logging level."""
        level = logging.DEBUG
        if self.tcex.default_args.logging is not None:
            level = logging.getLevelName(self.tcex.default_args.logging.upper())
        elif self.tcex.default_args.tc_log_level is not None:
            level = logging.getLevelName(self.tcex.default_args.tc_log_level.upper())
        return level

    def remove_handler_by_name(self, handler_name):
        """Remove a file handler by name."""
        for h in self._logger.handlers:
            if h.get_name() == handler_name:
                self._logger.removeHandler(h)
                break

    def update_handler_level(self, level=None):
        """Remove a file handler by name."""
        updated_level = self.logging_level
        if level is not None:
            updated_level = logging.getLevelName(level.upper())
        for h in self._logger.handlers:
            h.setLevel(updated_level)

    #
    # handlers
    #

    def add_api_handler(self, name='api'):
        """Add API logging handler."""
        self.remove_handler_by_name(name)
        api = ApiHandler(self.tcex.session)
        api.set_name(name)
        api.setLevel(self.logging_level)
        api.setFormatter(ApiFormatter())
        self._logger.addHandler(api)

    def add_file_handler(self, name='fh', filename=None, formatter=None, level=None):
        """Add File logging handler."""
        self.remove_handler_by_name(name)
        filename = filename or self.tcex.default_args.tc_log_file
        formatter = formatter or FileHandleFormatter()
        level = level or self.logging_level
        # create customized handler
        fh = FileHandlerCustom(os.path.join(self.tcex.default_args.tc_log_path, filename))
        fh.set_name(name)
        fh.setFormatter(formatter)
        fh.setLevel(level)
        self._logger.addHandler(fh)

    def add_rotating_file_handler(
        self,
        name='rfh',
        filename=None,
        level=None,
        formatter=None,
        backup_count=None,
        max_bytes=None,
        mode=None,
    ):
        """Return rotating file handler."""
        self.remove_handler_by_name(name)
        backup_count = backup_count or self.tcex.default_args.tc_log_backup_count
        filename = filename or self.tcex.default_args.tc_log_file
        formatter = formatter or FileHandleFormatter()
        level = level or self.logging_level
        max_bytes = max_bytes or self.tcex.default_args.tc_log_max_bytes
        mode = mode or 'a'
        # create customized handler
        fh = RotatingFileHandlerCustom(
            os.path.join(self.tcex.default_args.tc_log_path, filename),
            backupCount=backup_count,
            maxBytes=max_bytes,
            mode=mode,
        )
        fh.set_name(name)
        fh.setFormatter(formatter)
        fh.setLevel(level)
        self._logger.addHandler(fh)

    def add_stream_handler(self, name='sh', formatter=None, level=None):
        """Return stream logging handler."""
        self.remove_handler_by_name(name)
        formatter = formatter or FileHandleFormatter()
        level = level or self.logging_level
        # create handler
        sh = logging.StreamHandler()
        sh.set_name(name)
        sh.setFormatter(formatter)
        sh.setLevel(level)
        self._logger.addHandler(sh)

    #
    # App info logging
    #

    def log_info(self):
        """Send System and App data to logs."""
        self._log_platform()
        self._log_app_data()
        self._log_python_version()
        self._log_tcex_version()
        self._log_tc_proxy()

    def _log_app_data(self):
        """Log the App data information."""
        # Best Effort
        if self.tcex.install_json:
            app_commit_hash = self.tcex.install_json.get('commitHash')
            app_features = ','.join(self.tcex.install_json.get('features', []))
            app_min_ver = self.tcex.install_json.get('minServerVersion', 'N/A')
            app_name = self.tcex.install_json.get('displayName')
            app_runtime_level = self.tcex.install_json.get('runtimeLevel')
            app_version = self.tcex.install_json.get('programVersion')

            self.log.info('App Name: {}'.format(app_name))
            if app_features:
                self.log.info('App Features: {}'.format(app_features))
            self.log.info('App Minimum ThreatConnect Version: {}'.format(app_min_ver))
            self.log.info('App Runtime Level: {}'.format(app_runtime_level))
            self.log.info('App Version: {}'.format(app_version))
            if app_commit_hash is not None:
                self.log.info('App Commit Hash: {}'.format(app_commit_hash))

    def _log_platform(self):
        """Log the current Platform."""
        self.log.info('Platform: {}'.format(platform.platform()))

    def _log_python_version(self):
        """Log the current Python version."""
        self.log.info(
            'Python Version: {}.{}.{}'.format(
                sys.version_info.major, sys.version_info.minor, sys.version_info.micro
            )
        )

    def _log_tc_proxy(self):
        """Log the proxy settings."""
        if self.tcex.default_args.tc_proxy_tc:
            self.log.info(
                'Proxy Server (TC): {}:{}.'.format(
                    self.tcex.default_args.tc_proxy_host, self.tcex.default_args.tc_proxy_port
                )
            )

    def _log_tcex_version(self):
        """Log the current TcEx version number."""
        self.log.info('TcEx Version: {}'.format(__import__(__name__).__version__))


class ApiFormatter(logging.Formatter):
    """Logger formatter for ThreatConnect Exchange API logging."""

    def __init__(self, task_name=None):
        """Initialize Class properties."""
        self.task_name = task_name
        super(ApiFormatter, self).__init__()

    def format(self, record):
        """Format log record for ThreatConnect API.

        Example log event::

            [{
                "timestamp": 1478907537000,
                "message": "Test Message",
                "level": "DEBUG"
            }]
        """
        return {
            'timestamp': int(float(record.created or time.time()) * 1000),
            'message': record.msg or '',
            'level': record.levelname or 'DEBUG',
        }


class ApiHandler(logging.Handler):
    """Logger handler for ThreatConnect Exchange API logging."""

    def __init__(self, session, flush_limit=100):
        """Initialize Class properties.

        Args:
            session (Request.Session): The preconfigured instance of Session for ThreatConnect API.
            flush_limit (int): The limit to flush batch logs to the API.
        """
        super(ApiHandler, self).__init__()
        self.session = session
        self.flush_limit = flush_limit
        self.entries = []

    def close(self):
        """Close the logger and flush entries."""
        self.log_to_api()
        self.entries = []

    def emit(self, record):
        """Emit the log record."""
        self.entries.append(self.format(record))
        if len(self.entries) > self.flush_limit and not self.session.auth.renewing:
            self.log_to_api()
            self.entries = []

    def log_to_api(self):
        """Best effort API logger.

        Send logs to the ThreatConnect API and do nothing if the attempt fails.
        """
        if self.entries:
            try:
                headers = {'Content-Type': 'application/json'}
                self.session.post('/v2/logs/app', headers=headers, json=self.entries)
                # self.entries = []  # clear entries
            except Exception:
                # best effort on api logging
                pass


class FileHandlerCustom(logging.FileHandler):
    """Logger handler for ThreatConnect Exchange File logging."""

    def __init__(self, filename, mode='a', encoding=None, delay=0):
        """Add logic to create log directory if it does not exists."""
        if not os.path.exists(os.path.dirname(filename)):
            os.makedirs(os.path.dirname(filename), exist_ok=True)
        logging.FileHandler.__init__(self, filename, mode, encoding, delay)


class FileHandleFormatter(logging.Formatter):
    """Logger formatter for ThreatConnect Exchange file handler logging."""

    standard_format = (
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s '
        '(%(filename)s:%(funcName)s:%(lineno)d:%(thread)d)'
    )
    trace_format = (
        '%(asctime)s - %(name)s - %(levelname)s - [%(funcName)s:%(lineno)d] %(message)s '
        '(%(filename)s:%(thread)d)'
    )

    def __init__(self):
        """Initialize formatter parent."""
        try:  # py3
            super(FileHandleFormatter, self).__init__(
                fmt='%(levelno)d: %(msg)s', datefmt=None, style='%'
            )
        except TypeError:  # py2
            # TODO: remove after py2 support is dropped
            super(FileHandleFormatter, self).__init__(fmt='%(levelno)d: %(msg)s', datefmt=None)

    def format(self, record):
        """Format file handle log event according to logging level."""

        # Save the original format configured by the user
        # when the logger formatter was instantiated
        format_orig = self._style._fmt

        # Replace the original format with one customized by logging level
        if record.levelno in [logging.DEBUG, logging.TRACE]:
            self._style._fmt = FileHandleFormatter.trace_format
        else:
            self._style._fmt = FileHandleFormatter.standard_format

        # Call the original formatter class to do the grunt work
        result = logging.Formatter.format(self, record)

        # Restore the original format configured by the user
        self._style._fmt = format_orig

        return result


class RotatingFileHandlerCustom(RotatingFileHandler):
    """Logger handler for ThreatConnect Exchange File logging."""

    def __init__(self, filename, mode='a', maxBytes=0, backupCount=0, encoding=None, delay=0):
        """Add logic to create log directory if it does not exists."""
        if not os.path.exists(os.path.dirname(filename)):
            os.makedirs(os.path.dirname(filename), exist_ok=True)
        RotatingFileHandler.__init__(self, filename, mode, maxBytes, backupCount, encoding, delay)

    # TODO: investigate controlling logging thread events
    # def emit(self, record):
    #     """Emit the log record."""
    #     self.reopenIfNeeded()
    #     print(self.name)
    #     logging.FileHandler.emit(self, record)



# -*- coding: utf-8 -*-
"""ThreatConnect TI Report"""
from tcex.tcex_ti.mappings.group.tcex_ti_group import Group


class Report(Group):
    """Unique API calls for Report API Endpoints"""

    def __init__(self, tcex, name, owner=None, **kwargs):
        """Initialize Class Properties.

        Valid status:
        + Success
        + Awaiting Upload
        + In Progress
        + Failed

        Args:
            name (str): The name for this Group.
            date_added (str, kwargs): The date timestamp the Indicator was created.
            file_name (str, kwargs): The name for the attached file for this Group.
            file_content (str;method, kwargs): The file contents or callback method to retrieve
                                               file content.
            publish_date (str, kwargs): The publish datetime expression for this Group.
        """
        super(Report, self).__init__(
            tcex, 'Report', 'report', 'reports', owner=owner, name=name, **kwargs
        )

    def file_content(self, file_content, update_if_exists=True):
        """
        Updates the file content.

        Args:
            file_content: The file_content to upload.
            update_if_exists:

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['fileContent'] = file_content
        return self.tc_requests.upload(
            self.api_type,
            self.api_branch,
            self.unique_id,
            file_content,
            update_if_exists=update_if_exists,
        )

    def file_name(self, file_name):
        """
        Updates the file_name.

        Args:
            file_name:
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['fileName'] = file_name
        request = {'fileName': file_name}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)

    def file_size(self, file_size):
        """
        Updates the file_size.

        Args:
            file_size:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['fileSize'] = file_size
        request = {'fileSize': file_size}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)

    def status(self, status):
        """
        Updates the status

        Valid status:
        + Success
        + Awaiting Upload
        + In Progress
        + Failed

        Args:
            status: Success, Awaiting Upload, In Progress, or Failed
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['status'] = status
        request = {'status': status}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)

    def malware(self, malware, password, file_name):
        """
        Uploads to malware vault.

        Args:
            malware:
            password:
            file_name:
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        self._data['malware'] = malware
        self._data['password'] = password
        self._data['fileName'] = file_name
        request = {'malware': malware, 'password': password, 'fileName': file_name}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)

    def publish_date(self, publish_date):
        """Return Email to.
        :param publish_date:
        :return:
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        publish_date = self._utils.format_datetime(publish_date, date_format='%Y-%m-%dT%H:%M:%SZ')

        self._data['publishDate'] = publish_date
        request = {'publishDate': publish_date}
        return self.tc_requests.update(self.api_type, self.api_branch, self.unique_id, request)

    def download(self):
        """
        Downloads the documents context.

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.download(self.api_type, self.api_branch, self.unique_id)

    def get_file_hash(self, hash_type='sha256'):
        """
        Getting the hash value of attached document
        Args:
            hash_type:

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.get_file_hash(
            self.api_type, self.api_branch, self.unique_id, hash_type=hash_type
        )



# -*- coding: utf-8 -*-
"""ThreatConnect TI Indicator"""
import json

try:
    from urllib import quote_plus  # Python 2
    from urllib import unquote  # Python 2
except ImportError:
    from urllib.parse import quote_plus  # Python
    from urllib.parse import unquote  # Python

from tcex.tcex_ti.mappings.tcex_ti_mappings import TIMappings

# import local modules for dynamic reference
module = __import__(__name__)

# custom_class = custom_indicator_class_factory(
#     entry.get('name'),
#     entry.get('apiEntity'),
#     entry.get('apiBranch'),
#     Indicator,
#     class_data,
#     value_fields,
# )


def custom_indicator_class_factory(
    indicator_type, entity_type, branch_type, base_class, value_fields
):
    """Internal method for dynamically building Custom Indicator Class."""
    value_count = len(value_fields)

    def init_1(self, tcex, value1, owner=None, **kwargs):  # pylint: disable=W0641
        """Init method for Custom Indicator Types with one value
        :param self:
        :param tcex:
        :param owner:
        :param value1:
        :param kwargs:
        """
        base_class.__init__(self, tcex, indicator_type, entity_type, branch_type, owner, **kwargs)
        self.unique_id = kwargs.get('unique_id', value1)
        self._data[value_fields[0]] = value1 or self.unique_id
        if self.unique_id:
            self.unique_id = quote_plus(self.fully_decode_uri(self.unique_id))

    def _set_unique_id_1(self, json_request):
        """

        :param self:
        :param json_request:
        """
        self.unique_id = json_request.get(value_fields[0])
        if self.unique_id:
            self.unique_id = quote_plus(self.fully_decode_uri(self.unique_id))

    def can_create_1(self):  # pylint: disable=W0641
        """
        Determines if the required data that the API endpoint is expecting is present.
        :return: Boolean
        """
        if self.data.get(value_fields[0]):
            return True
        return False

    def init_2(self, tcex, value1, value2, owner=None, **kwargs):  # pylint: disable=W0641
        """Init method for Custom Indicator Types with two values.
        :param self:
        :param tcex:
        :param value1:
        :param value2:
        :param owner:
        :param kwargs:
        """
        base_class.__init__(self, tcex, indicator_type, entity_type, branch_type, owner, **kwargs)
        self._data[value_fields[0]] = value1
        self._data[value_fields[1]] = value2
        if value1:
            value1 = quote_plus(self.fully_decode_uri(value1))
        if value2:
            value2 = quote_plus(self.fully_decode_uri(value2))
        self.unique_id = kwargs.get('unique_id', self.build_summary(value1, value2))

    def _set_unique_id_2(self, json_request):
        """

        :param self:
        :param json_request:
        """
        value_0 = json_request.get(value_fields[0], '')
        value_1 = json_request.get(value_fields[1], '')
        self.unique_id = self.build_summary(
            quote_plus(self.fully_decode_uri(value_0)) or None,
            quote_plus(self.fully_decode_uri(value_1)) or None,
        )

    def can_create_2(self):  # pylint: disable=W0641
        """
        Determines if the required data that the API endpoint is expecting is present.
        :return: Boolean
        """
        if self.data.get(value_fields[0]) and self.data.get(value_fields[1]):
            return True
        return False

    def init_3(self, tcex, value1, value2, value3, owner=None, **kwargs):  # pylint: disable=W0641
        """Init method for Custom Indicator Types with three values.
        :param self:
        :param tcex:
        :param value1:
        :param value2:
        :param value3:
        :param kwargs:
        """
        base_class.__init__(self, tcex, indicator_type, entity_type, branch_type, owner, **kwargs)
        self._data[value_fields[0]] = value1
        self._data[value_fields[1]] = value2
        self._data[value_fields[2]] = value3
        if value1:
            value1 = quote_plus(self.fully_decode_uri(value1))
        if value2:
            value2 = quote_plus(self.fully_decode_uri(value2))
        if value3:
            value3 = quote_plus(self.fully_decode_uri(value3))
        self.unique_id = kwargs.get('unique_id', self.build_summary(value1, value2, value3))

    def _set_unique_id_3(self, json_request):
        """

        :param self:
        :param json_request:
        """
        value_0 = json_request.get(value_fields[0], '')
        value_1 = json_request.get(value_fields[1], '')
        value_2 = json_request.get(value_fields[2], '')
        self.unique_id = self.build_summary(
            quote_plus(self.fully_decode_uri(value_0)) or None,
            quote_plus(self.fully_decode_uri(value_1)) or None,
            quote_plus(self.fully_decode_uri(value_2)) or None,
        )

    def can_create_3(self):  # pylint: disable=W0641
        """
        Determines if the required data that the API endpoint is expecting is present.
        :return: Boolean
        """
        if (
            self.data.get(value_fields[0])
            and self.data.get(value_fields[1])
            and self.data.get(value_fields[2])
        ):
            return True
        return False

    class_name = indicator_type.replace(' ', '')
    init_method = locals()['init_{}'.format(value_count)]
    set_unique_id_method = locals()['_set_unique_id_{}'.format(value_count)]
    can_create_method = locals()['can_create_{}'.format(value_count)]
    new_class = type(
        str(class_name),
        (base_class,),
        {
            '__init__': init_method,
            '_set_unique_id': set_unique_id_method,
            'can_create': can_create_method,
        },
    )
    return new_class


class Indicator(TIMappings):
    """Unique API calls for Indicator API Endpoints"""

    def __init__(self, tcex, sub_type, api_entity, api_branch, owner, **kwargs):
        super(Indicator, self).__init__(
            tcex, 'Indicator', 'indicators', sub_type, api_entity, api_branch, owner
        )

        for arg, value in kwargs.items():
            self.add_key_value(arg, value)

    @staticmethod
    def is_indicator():
        return True

    @property
    def owner(self):
        return self._owner

    @staticmethod
    def is_encoded(uri):
        """Determines if a uri is currently encoded"""
        uri = uri or ''

        return uri != unquote(uri)

    def fully_decode_uri(self, uri):
        """Decodes a url till it is no longer encoded."""
        saftey_valve = 0

        while self.is_encoded(uri):
            uri = unquote(uri)
            saftey_valve += 1
            if saftey_valve > 10:
                break

        return uri

    def can_create(self):
        """
        Overridden by other indicator classes.

        Returns:

         """
        return True

    @property
    def _metadata_map(self):
        return {
            'date_added': 'dateAdded',
            'dns_active': 'dnsActive',
            'last_modified': 'lastModified',
            'private_flag': 'privateFlag',
            'size': 'intValue1',
            'whois_active': 'whoisActive',
            'key_name': 'Key Name',
            'value_type': 'Value Type',
            'value_name': 'Value Name',
            'user_agent_string': 'User Agent String',
            'block': 'Block',
            'mutex': 'Mutex',
            'as_number': 'AS Number',
        }

    def add_key_value(self, key, value):
        """
        Converts the value and adds it as a data field.

        Args:
            key:
            value:
        """
        key = self._metadata_map.get(key, key)
        if key in ['dateAdded', 'lastModified']:
            self._data[key] = self._utils.format_datetime(value, date_format='%Y-%m-%dT%H:%M:%SZ')
        elif key == 'confidence':
            self._data[key] = int(value)
        elif key == 'rating':
            self._data[key] = float(value)
        elif key == 'unique_id':
            self._unique_id = quote_plus(self.fully_decode_uri(value))
        else:
            self._data[key] = value

    def rating(self, value):
        """
        Updates the Indicators rating

        Args:
            value:
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])
        request_data = {'rating': value}
        return self.tc_requests.update(
            self.api_type, self.api_branch, self.unique_id, request_data, owner=self.owner
        )

    def confidence(self, value):
        """
        Updates the Indicators confidence

        Args:
            value:
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])
        request_data = {'confidence': value}
        return self.tc_requests.update(
            self.api_type, self.api_branch, self.unique_id, request_data, owner=self.owner
        )

    def owners(self):
        """

        :return:
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])
        return self.tc_requests.owners(
            self.api_type, self.api_branch, self.unique_id, owner=self.owner
        )

    def add_observers(self, count, date_observed):
        """
        Adds a Indicator Observation

        Args:
            count:
            date_observed:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        data = {
            'count': count,
            'dataObserved': self._utils.format_datetime(
                date_observed, date_format='%Y-%m-%dT%H:%M:%SZ'
            ),
        }

        return self.tc_requests.add_observations(
            self.api_type, self.api_branch, self.unique_id, data, owner=self.owner
        )

    def observation_count(self):
        """
        Gets the indicators observation count.

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])
        return self.tc_requests.observation_count(
            self.api_type, self.api_branch, self.unique_id, owner=self.owner
        )

    def add_false_positive(self):
        """
        Adds a Indicator FalsePositive
        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])

        return self.tc_requests.add_false_positive(
            self.api_type, self.api_branch, self.unique_id, owner=self.owner
        )

    def observations(self):
        """
        Gets the indicators observations.

        Returns:

        """
        if not self.can_update():
            self._tcex.handle_error(910, [self.type])
        return self.tc_requests.observations(
            self.api_type, self.api_branch, self.unique_id, owner=self.owner
        )

    def deleted(self, deleted_since, filters=None, params=None):
        """
        Gets the indicators deleted.

        Args:
            params:
            filters:
            deleted_since: Date since its been deleted

        """

        return self.tc_requests.deleted(
            self.api_type,
            self.api_branch,
            deleted_since,
            owner=self.owner,
            filters=filters,
            params=params,
        )

    @staticmethod
    def build_summary(val1=None, val2=None, val3=None):
        """
        Constructs the summary given va1, va2, val3

        Args:
            val1:
            val2:
            val3:

        Returns:

        """
        summary = []
        if val1 is not None:
            summary.append(val1)
        if val2 is not None:
            summary.append(val2)
        if val3 is not None:
            summary.append(val3)
        if not summary:
            return None
        return ' : '.join(summary)

    def __str__(self):
        """Return string represtentation of object"""
        return json.dumps(self._data, indent=4)



# -*- coding: utf-8 -*-
"""Test the TcEx Threat Intel Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestVictim:
    """Test TcEx Victim."""

    def setup_class(self):
        """Configure setup before all tests."""
        self.ti = tcex.ti

    def test_victim_get(self, victim_id=None, name='victim-name-42353'):
        """Test victim get."""
        # create
        if not victim_id:
            victim_id = self.victim_create(name)

        # get
        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.single()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get(ti.api_entity).get('name') == name

        # delete
        self.victim_delete()

    def test_victim_get_attributes(self, victim_id=None, name='victim-name-12453'):
        """Test victim get."""
        # create
        if not victim_id:
            victim_id = self.victim_create(name)

        self.test_victim_add_attribute(
            victim_id=victim_id, attribute_type='Description', attribute_value='test1'
        )
        self.test_victim_add_attribute(
            victim_id=victim_id, attribute_type='Description', attribute_value='test2'
        )
        self.test_victim_add_attribute(
            victim_id=victim_id, attribute_type='Description', attribute_value='test3'
        )

        # get attributes
        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        for attribute in ti.attributes():
            assert attribute
            break
        else:
            assert False

        # delete
        self.victim_delete()

    def test_victim_get_tags(self, victim_id=None, name='victim-name-64235'):
        """Test victim get."""
        # create
        if not victim_id:
            victim_id = self.victim_create(name)

        self.test_victim_add_tag(victim_id=victim_id, tag='One')
        self.test_victim_add_tag(victim_id=victim_id, tag='Two')

        # get tags
        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        for tag in ti.tags():
            assert tag.get('name')
            break
        else:
            assert False

        # delete
        self.victim_delete()

    def test_victim_get_include(self, victim_id=None, name='victim-name-78159'):
        """Test victim get."""
        if not victim_id:
            victim_id = self.victim_create(name)

        self.test_victim_add_attribute(
            victim_id=victim_id, attribute_type='Description', attribute_value='test123'
        )
        self.test_victim_add_label(victim_id=victim_id, label='TLP:RED')
        self.test_victim_add_tag(victim_id=victim_id, tag='PyTest')

        parameters = {'includes': ['additional', 'attributes', 'labels', 'tags']}
        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.single(params=parameters)

        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('victim').get('name') == name
        assert ti_data.get('data').get('victim').get('attribute')[0].get('value') == 'test123'
        # assert ti_data.get('data').get('victim').get('securityLabel')[0].get('name') == 'TLP:RED'
        assert ti_data.get('data').get('victim').get('tag')[0].get('name') == 'PyTest'

        # delete
        self.victim_delete()

    def victim_create(self, name='victim-name-65341'):
        """Test victim create."""
        ti = self.ti.victim(name, owner=tcex.args.tc_owner)
        r = ti.create()
        ti_data = r.json()
        assert r.status_code == 201
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('victim').get('name') == name
        return ti.unique_id

    def test_victim_add_attribute(
        self,
        victim_id=None,
        name='victim-name-nkjvb',
        attribute_type='Description',
        attribute_value='Example Description.',
    ):
        """Test victim attribute add."""
        should_delete = False
        if not victim_id:
            should_delete = True
            victim_id = self.victim_create(name)

        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.add_attribute(attribute_type=attribute_type, attribute_value=attribute_value)
        attribute_data = r.json()
        assert r.status_code == 201
        assert attribute_data.get('status') == 'Success'
        assert attribute_data.get('data').get('attribute').get('value') == attribute_value

        if should_delete:
            self.victim_delete()

    def test_victim_add_label(self, victim_id=None, name='victim-name-ds4vb', label='TLP:GREEN'):
        """Test victim attribute add."""
        should_delete = False
        if not victim_id:
            should_delete = True
            victim_id = self.victim_create(name)

        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.add_label(label=label)
        label_data = r.json()
        assert r.status_code == 201
        assert label_data.get('status') == 'Success'

        if should_delete:
            self.victim_delete()

    def test_victim_add_tag(self, victim_id=None, name='victim-name-fdsv23', tag='Crimeware'):
        """Test victim attribute add."""
        should_delete = False
        if not victim_id:
            should_delete = True
            victim_id = self.victim_create(name)

        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.add_tag(tag)
        tag_data = r.json()
        assert r.status_code == 201
        assert tag_data.get('status') == 'Success'

        if should_delete:
            self.victim_delete()

    def victim_delete(self, victim_id=None, name='victim-name-bdsfd'):
        """Test victim delete."""
        # create indicator
        if not victim_id:
            victim_id = self.victim_create(name)

        # delete indicator
        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.delete()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'

    def test_victim_update(self, victim_id=None, name='victim-name-b3da3'):
        """Test victim update."""
        # create indicator
        if not victim_id:
            victim_id = self.victim_create(name)

        name = 'victim-new-name-fdasb3'

        # update indicator
        ti = self.ti.victim(name, owner=tcex.args.tc_owner, unique_id=victim_id)
        r = ti.update()
        ti_data = r.json()
        assert r.status_code == 200
        assert ti_data.get('status') == 'Success'
        assert ti_data.get('data').get('victim').get('name') == name

        # delete indicator
        self.victim_delete()



# -*- coding: utf-8 -*-
"""Test the TcEx Metrics Module."""

from ..tcex_init import tcex


# pylint: disable=W0201
class TestMetrics:
    """Test the TcEx Metrics Module."""

    def setup_class(self):
        """Configure setup before all tests."""

    @staticmethod
    def test_metrics():
        """Test metrics."""
        date = '2008-12-12T12:12:12'
        result_date = '2008-12-12T00:00:00Z'
        metrics = tcex.metric(
            name='pytest metrics',
            description='pytest',
            data_type='Sum',
            interval='Daily',
            keyed=False,
        )
        results = metrics.add(value=42, date=date, return_value=True)
        assert results.get('date') == result_date

    @staticmethod
    def test_metrics_keyed():
        """Test key metrics."""
        date = '2008-12-12T12:12:12'
        result_date = '2008-12-12T00:00:00Z'
        metrics = tcex.metric(
            name='pytest metrics by Owner',
            description='pytest',
            data_type='Sum',
            interval='Daily',
            keyed=True,
        )
        results = metrics.add_keyed(value=42, key='MyOrg', date=date, return_value=True)
        assert results.get('date') == result_date



# -*- coding: utf-8 -*-
"""ThreatConnect TI Security Label"""

# import local modules for dynamic reference
from tcex.tcex_ti.tcex_ti_tc_request import TiTcRequest
from tcex.tcex_utils import TcExUtils


class Tag(object):
    """Unique API calls for Tag API Endpoints"""

    def __init__(self, tcex, name):
        """Initialize Class Properties.

        Args:
            group_type (str): The ThreatConnect define Group type.
            name (str): The name for this Group.
            xid (str, kwargs): The external id for this Group.
        """
        self._name = name
        self._tcex = tcex
        self._type = 'tags'
        self._api_sub_type = None
        self._api_type = None
        self._api_entity = 'tag'

        self._utils = TcExUtils()
        self._tc_requests = TiTcRequest(self._tcex)

    @staticmethod
    def is_tag():
        """
        Indicates that this is a tag object
        Returns:

        """
        return True

    def groups(self, group_type=None, filters=None, params=None):
        """
        Gets all groups from a tag.

        Args:
            filters:
            params:
            group_type:
        """
        group = self._tcex.ti.group(group_type)
        for g in self.tc_requests.groups_from_tag(group, self.name, filters=filters, params=params):
            yield g

    def indicators(self, indicator_type=None, filters=None, params=None):
        """
        Gets all indicators from a tag.

        Args:
            params:
            filters:
            indicator_type:
        """
        indicator = self._tcex.ti.indicator(indicator_type)
        for i in self.tc_requests.indicators_from_tag(
            indicator, self.name, filters=filters, params=params
        ):
            yield i

    def victims(self, filters=None, params=None):
        """
        Gets all victims from a tag.
        """
        victim = self._tcex.ti.victim(None)
        for v in self.tc_requests.victims_from_tag(
            victim, self.name, filters=filters, params=params
        ):
            yield v

    @property
    def name(self):
        """
        Gets the tag name.
        """
        return self._name

    @property
    def tc_requests(self):
        """
        Gets the tc request object
        """
        return self._tc_requests

    @name.setter
    def name(self, name):
        """
        Sets the tag name

        Args:
            name:

        Returns:

        """
        self._name = name

    @tc_requests.setter
    def tc_requests(self, tc_requests):
        """
        Sets the tc request object.

        Args:
            tc_requests:

        Returns:

        """
        self._tc_requests = tc_requests



# -*- coding: utf-8 -*-
"""TcEx Testing Initialization."""
import os

from tcex import TcEx

# a token in required for DataStore testing
if not os.getenv('TC_TOKEN'):
    raise RuntimeError('A Token is required to run tests.')

config_data = {
    # connection
    'api_default_org': os.getenv('API_DEFAULT_ORG'),
    'tc_token': os.getenv('TC_TOKEN'),
    'tc_token_expires': os.getenv('TC_TOKEN_EXPIRES'),
    'tc_owner': os.getenv('TC_OWNER', 'TCI'),
    # logging
    'tc_log_level': os.getenv('TC_LOG_LEVEL', 'trace'),
    'tc_log_to_api': str(os.getenv('TC_LOG_TO_API', 'false')).lower() in ['true'],
    # paths
    'tc_api_path': os.getenv('TC_API_PATH'),
    'tc_in_path': os.getenv('TC_IN_PATH', 'log'),
    'tc_log_path': os.getenv('TC_LOG_PATH', 'log'),
    'tc_out_path': os.getenv('TC_OUT_API', 'log'),
    'tc_temp_path': os.getenv('TC_TEMP_PATH', 'log'),
    # playbooks
    'tc_playbook_db_type': os.getenv('TC_PLAYBOOK_DB_TYPE', 'Redis'),
    'tc_playbook_db_context': os.getenv(
        'TC_PLAYBOOK_DB_CONTEXT', '0d5a675a-1d60-4679-bd01-3948d6a0a8bd'
    ),
    'tc_playbook_db_path': os.getenv('TC_PLAYBOOK_DB_PATH', 'localhost'),
    'tc_playbook_db_port': os.getenv('TC_PLAYBOOK_DB_PORT', '6379'),
    # proxy
    'tc_proxy_tc': str(os.getenv('TC_PROXY_TC', 'false')).lower() in ['true'],
    'tc_proxy_external': str(os.getenv('TC_PROXY_EXTERNAL', 'false')).lower() in ['true'],
}

# proxy
if os.getenv('TC_PROXY_HOST'):
    config_data['tc_proxy_host'] = os.getenv('TC_PROXY_HOST')
if os.getenv('TC_PROXY_PORT'):
    config_data['tc_proxy_port'] = os.getenv('TC_PROXY_PORT')
if os.getenv('TC_PROXY_USERNAME'):
    config_data['tc_proxy_username'] = os.getenv('TC_PROXY_USERNAME')
if os.getenv('TC_PROXY_PASSWORD'):
    config_data['tc_proxy_password'] = os.getenv('TC_PROXY_PASSWORD')

tcex = TcEx()
tcex.tcex_args.config(config_data)
args = tcex.args
